Package: geoAr
Title: Argentina's Spatial Data Toolbox
Version: 1.2.2
Authors@R: 
   c( person(given = "Juan Pablo",
           family = "Ruiz Nicolini",
           role = c("aut", "cre", "cph"),
           email = "juanpabloruiznicolini@gmail.com",
           comment = c(ORCID = "0000-0002-3138-6343")), 
      person(given = "Patricio",
           family = "Del Boca",
           role = c("aut"),
           email = "patriciodelboca@gmail.com"),
      person(given = "Juan Gabriel",
           family = "Juara",
           role = c("aut"),
           email = "jg.juara@gmail.com"))
Description: Collection of tools that facilitates data access and workflow for spatial analysis of Argentina. Includes historical information from censuses, administrative limits at different levels of aggregation, location of human settlements, among others. Since it is expected that the majority of users will be Spanish-speaking, the documentation of the package prioritizes this language, although an effort is made to also offer annotations in English. 
License: MIT + file LICENSE
Language: es, en
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.2
Suggests: testthat, gt, knitr, rmarkdown, geofacet, ggplot2, tibble
Depends: R (>= 4.1.0)
Imports: dplyr, sf, httr2 (>= 1.0.0), promises (>= 1.2.0), assertthat,
        attempt, tidyr, stringr, magrittr, curl, glue, leaflet,
        jsonlite, purrr
URL: https://politicaargentina.github.io/geoAr/,
        https://github.com/PoliticaArgentina/geoAr
BugReports: https://github.com/PoliticaArgentina/geoAr/issues
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-05-07 15:01:36 UTC; UsuarioFundar
Author: Juan Pablo Ruiz Nicolini [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-3138-6343>),
  Patricio Del Boca [aut],
  Juan Gabriel Juara [aut]
Maintainer: Juan Pablo Ruiz Nicolini <juanpabloruiznicolini@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-08 08:20:03 UTC
Built: R 4.6.0; ; 2026-05-08 10:26:40 UTC; unix

Package: geoAr
Title: Argentina's Spatial Data Toolbox
Version: 1.2.2
Authors@R: 
   c( person(given = "Juan Pablo",
           family = "Ruiz Nicolini",
           role = c("aut", "cre", "cph"),
           email = "juanpabloruiznicolini@gmail.com",
           comment = c(ORCID = "0000-0002-3138-6343")), 
      person(given = "Patricio",
           family = "Del Boca",
           role = c("aut"),
           email = "patriciodelboca@gmail.com"),
      person(given = "Juan Gabriel",
           family = "Juara",
           role = c("aut"),
           email = "jg.juara@gmail.com"))
Description: Collection of tools that facilitates data access and workflow for spatial analysis of Argentina. Includes historical information from censuses, administrative limits at different levels of aggregation, location of human settlements, among others. Since it is expected that the majority of users will be Spanish-speaking, the documentation of the package prioritizes this language, although an effort is made to also offer annotations in English. 
License: MIT + file LICENSE
Language: es, en
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.2
Suggests: testthat, gt, knitr, rmarkdown, geofacet, ggplot2, tibble
Depends: R (>= 4.1.0)
Imports: dplyr, sf, httr2 (>= 1.0.0), promises (>= 1.2.0), assertthat,
        attempt, tidyr, stringr, magrittr, curl, glue, leaflet,
        jsonlite, purrr
URL: https://politicaargentina.github.io/geoAr/,
        https://github.com/PoliticaArgentina/geoAr
BugReports: https://github.com/PoliticaArgentina/geoAr/issues
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-05-07 15:01:36 UTC; UsuarioFundar
Author: Juan Pablo Ruiz Nicolini [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-3138-6343>),
  Patricio Del Boca [aut],
  Juan Gabriel Juara [aut]
Maintainer: Juan Pablo Ruiz Nicolini <juanpabloruiznicolini@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-08 08:20:03 UTC
Built: R 4.6.0; ; 2026-05-08 11:25:14 UTC; unix

# Generated by roxygen2: do not edit by hand

export("%>%")
export(addArgTiles)
export(add_geo_codes)
export(get_asentamientos)
export(get_bahra)
export(get_calles)
export(get_censo)
export(get_departamentos)
export(get_eph)
export(get_geo)
export(get_geodata_dump)
export(get_grid)
export(get_localidades)
export(get_localidades_censales)
export(get_municipios)
export(get_provincias)
export(get_ubicacion)
export(normalizar_direccion)
export(post_asentamientos_bulk)
export(post_calles_bulk)
export(post_departamentos_bulk)
export(post_direcciones_bulk)
export(post_localidades_bulk)
export(post_localidades_censales_bulk)
export(post_municipios_bulk)
export(post_provincias_bulk)
export(post_ubicacion_bulk)
export(recode_grid)
export(show_arg_codes)
export(show_available_censos)
importFrom(httr2,req_auth_bearer_token)
importFrom(httr2,req_body_json)
importFrom(httr2,req_error)
importFrom(httr2,req_method)
importFrom(httr2,req_perform)
importFrom(httr2,req_perform_parallel)
importFrom(httr2,req_url_query)
importFrom(httr2,request)
importFrom(httr2,resp_body_json)
importFrom(httr2,resp_body_raw)
importFrom(httr2,resp_body_string)
importFrom(httr2,resp_content_type)
importFrom(httr2,resp_has_body)
importFrom(httr2,resp_status)
importFrom(httr2,resp_status_desc)
importFrom(httr2,resp_url)
importFrom(magrittr,"%>%")
importFrom(promises,then)

## `geo`grafía `Ar`rgentina usando `R` <a><img src="man/figures/logo.png" width="200" align="right"/></a>

<!-- README.md is generated from README.Rmd. Please edit that file -->
<!-- badges: start -->

[![](https://www.r-pkg.org/badges/version/geoAr?color=blue)](https://cran.r-project.org/package=geoAr)
[![](http://cranlogs.r-pkg.org/badges/last-month/geoAr?color=blue)](https://cran.r-project.org/package=geoAr)[![](http://cranlogs.r-pkg.org/badges/last-week/geoAr?color=blue)](https://cran.r-project.org/package=geoAr)[![](http://cranlogs.r-pkg.org/badges/last-month/geoAr?color=blue)](https://cran.r-project.org/package=geoAr)[![](http://cranlogs.r-pkg.org/badges/grand-total/geoAr?color=blue)](https://cran.r-project.org/package=geoAr)
[![R build
status](https://github.com/PoliticaArgentina/geoAr/workflows/R-CMD-check/badge.svg)](https://github.com/PoliticaArgentina/geoAr/actions)

<!-- badges: end -->

`{geoAr}` brinda herramientas que facilitan el acceso y el trabajo con
datos espaciales de Argentina desde `R`.

------------------------------------------------------------------------

## INSTALACIÓN (*Install*)

### Desde `CRAN` (versión estable)

``` r

# Install geoAr in R:

install.packages('geoAr')
```

### Versión en desarrollo (*Development version*)

#### Desde *GitHub*

``` r
# install.packages('devtools') si no tiene instalado devtools

devtools::install_github("politicaargentina/geoAr")
```

#### `r-universe`

Se puede agregar el repositorio de
[`r-universe`](https://politicaargentina.r-universe.dev/builds) como
parámetro a la función base `install.packages`:

``` r

# Install geoAr in R:
install.packages('geoAr', repos = c('https://politicaargentina.r-universe.dev')
```

## Ejemplo de uso básico (*Basic Usage*)

``` r

library(geoAr)


show_arg_codes() # Preview avialable data 
#> # A tibble: 26 × 5
#>    id           codprov codprov_censo codprov_iso name_iso                      
#>    <chr>        <chr>   <chr>         <chr>       <chr>                         
#>  1 ARGENTINA    " "     " "           AR          Argentina                     
#>  2 CABA         "01"    "02"          AR-C        Ciudad Autónoma de Buenos Air…
#>  3 BUENOS AIRES "02"    "06"          AR-B        Buenos Aires                  
#>  4 CATAMARCA    "03"    "10"          AR-K        Catamarca                     
#>  5 CORDOBA      "04"    "14"          AR-X        Córdoba                       
#>  6 CORRIENTES   "05"    "18"          AR-W        Corrientes                    
#>  7 CHACO        "06"    "22"          AR-H        Chaco                         
#>  8 CHUBUT       "07"    "26"          AR-U        Chubut                        
#>  9 ENTRE RIOS   "08"    "30"          AR-E        Entre Ríos                    
#> 10 FORMOSA      "09"    "34"          AR-P        Formosa                       
#> # ℹ 16 more rows


(tucuman <- get_geo(geo = "TUCUMAN")) # geo id parameter from show_arg_codes() function
#> Simple feature collection with 17 features and 2 fields
#> Geometry type: MULTIPOLYGON
#> Dimension:     XY
#> Bounding box:  xmin: -66.18101 ymin: -28.01575 xmax: -64.48315 ymax: -26.06037
#> Geodetic CRS:  WGS 84
#> # A tibble: 17 × 3
#>    codprov_censo coddepto_censo                                         geometry
#>  * <chr>         <chr>                                        <MULTIPOLYGON [°]>
#>  1 90            007            (((-64.49919 -26.23353, -64.49857 -26.26325, -6…
#>  2 90            014            (((-65.13782 -26.74975, -65.10095 -26.75771, -6…
#>  3 90            021            (((-65.94729 -27.08444, -65.93885 -27.0989, -65…
#>  4 90            028            (((-65.62733 -26.87134, -65.62598 -26.84177, -6…
#>  5 90            035            (((-65.47532 -27.53838, -65.43503 -27.5425, -65…
#>  6 90            042            (((-65.51768 -27.54528, -65.53685 -27.62278, -6…
#>  7 90            049            (((-65.51348 -27.62978, -65.52091 -27.66553, -6…
#>  8 90            056            (((-65.12933 -27.02488, -65.12589 -27.01659, -6…
#>  9 90            063            (((-65.43583 -26.83751, -65.38124 -26.84196, -6…
#> 10 90            070            (((-65.86853 -26.99852, -65.85796 -27.01388, -6…
#> 11 90            077            (((-65.93481 -27.39544, -65.89887 -27.3702, -65…
#> 12 90            084            (((-65.16293 -26.82289, -65.18562 -26.86204, -6…
#> 13 90            091            (((-65.33499 -27.20478, -65.33462 -27.21948, -6…
#> 14 90            098            (((-66.08361 -26.23547, -66.06621 -26.23792, -6…
#> 15 90            105            (((-65.70111 -26.52317, -65.65078 -26.5263, -65…
#> 16 90            112            (((-65.52483 -26.09667, -65.4682 -26.09253, -65…
#> 17 90            119            (((-65.27025 -26.84087, -65.28587 -26.83549, -6…
```

## Información Disponible (*Available Data*)

### Geometrías

#### **Polígonos**

**Las funciones básicas para obtener polígonos de Argentina corresponden
a la última información censal disponible (CENSO 2010)**

- ARGENTINA (niveles PROVINCIA, DEPARTAMENTO y RADIO CENSAL)

- 24 provincias (niveles DEPARTAMENTO y RADIO CENSAL)

**CENSOS históricos**

- Se facilita el acceso a la reconstrucción de los polígonos utilizados
  para los censos desde 1869 hasta 2010. Recuperado del proyecto *Base
  cartográfica de departamentos para los censos nacionales de la
  República Argentina desde 1869 a 2010* de [Gonzalo Rodriguez y Pablo
  De Grande](https://ri.conicet.gov.ar/handle/11336/149867).

**Encuesta Permanente de Hogares (EPH)**

- Geometrías (entidades, envolventes y radios censales) correspondientes
  a la EPH del INDEC.

#### **Puntos (coordenadas)**

- Base de Asentamientos Humanos de la República Argentina (BAHRA)
  <http://www.bahra.gob.ar/>

### Grillas `geofacet`

- Grillas para usar con
  [geofacet](https://politicaargentina.github.io/geoAr/articles/geoAr_geofacet.html)
  para ARGENTINA, 24 provincias y AGLOMERADOS URBANOS (para usar con
  {eph}

- Libro de códigos (país, provincia, departamento) y función para
  re-codificar grillas según diccionario (ISO, INDEC, INDRA).

### `georef-ar` API

- Interfaz en R para usar la [API del Servicio de Normalización de Datos
  Geográficos](https://georef-ar-api.readthedocs.io/es/latest/), que
  permite normalizar y codificar los nombres de unidades territoriales
  de la Argentina (provincias, departamentos, municipios y localidades)
  y de sus calles, así como ubicar coordenadas dentro de ellas. La
  versión original de estas funciones fueron desarrolladas en el paquete
  `georefar` desarrollado por [Patricio del
  Boca](https://github.com/pdelboca/georefar).

## Paquetes similares para otros países (*Similar packages*)

- `{geoUy}` - <https://github.com/RichDeto/geouy>
- `{geoBr}` - <https://ipeagit.github.io/geobr/>
- `{chilemapas}` - <https://pacha.dev/chilemapas/>

## Citation

To cite package ‘geoAr’ in publications use:

*Ruiz Nicolini J, Del Boca P, Juara J (2024). geoAr: Argentina’s Spatial
Data Toolbox. R package version 1.0.0,
<https://github.com/PoliticaArgentina/geoAr>.*

A BibTeX entry for LaTeX users is


      
      @Manual{,
        title = {geoAr: Argentina's Spatial Data Toolbox},
        author = {Juan Pablo {Ruiz Nicolini} and Patricio {Del Boca} and Juan Gabriel Juara},
        year = {2024},
        note = {R package version 1.0.0},
        url = {https://github.com/PoliticaArgentina/geoAr},
      }
      

## `{geoAr}` es parte del [universo de paquetes **polAr**](https://github.com/PoliticaArgentina/polArverse)

<img src="https://github.com/PoliticaArgentina/data_warehouse/raw/master/hex/collage.png" width="100%" />

#' Descarga 'base total' de Asentamientos Humanos de la República Argentina (BAHRA)
#'
#' @param geo un character con el nombre del distrito que se quiere descargar (por defecto toda ARGENTINA)
#'Se puede chequear el id con \code{\link{show_arg_codes}}.
#' @return tibble con datos de BAHRA
#' @export
#'
#' @examples get_bahra()

get_bahra <- function(geo = "ARGENTINA"){

  ## Check for internet conection
  attempt::stop_if_not(.x = curl::has_internet(),
                       msg = "Internet access was not detected. Please check your connection //
No se detecto acceso a internet. Por favor chequear la conexion.")

  assertthat::assert_that(geo %in% c("ARGENTINA", "CABA", "CATAMARCA", "CHACO", "CHUBUT", "CORDOBA", "CORRIENTES",
                                     "ENTRE RIOS", "FORMOSA", "JUJUY", "LA PAMPA", "LA RIOJA", "MENDOZA", "MISIONES",
                                     "NEUQUEN","BUENOS AIRES", "RIO NEGRO", "SALTA", "SANTA CRUZ", "SANTA FE", "SANTIAGO DEL ESTERO",
                                     "SAN JUAN", "SAN LUIS", "TIERRA DEL FUEGO", "TUCUMAN"),
                          msg = "no es un geo valido. Chequearlos con show_arg_codes()")


  # Check parameters


  url <- "https://github.com/PoliticaArgentina/data_warehouse/raw/master/geoAr/data_raw/base_total_bahra.geojson"



  # Set default value for try()

  default <- NULL

  df <- base::suppressWarnings(base::try(default <- sf::read_sf(url), silent = TRUE))

  if(is.null(default)){

    df <- base::message("Fail to download data. Source is not available // La fuente de datos no esta disponible")

  } else {

    df <- df %>%
      dplyr::mutate(cod_depto = stringr::str_sub(cod_depto, start = 3, end = 5)) %>%
      dplyr::rename(codprov_censo = "cod_pcia",
                    coddepto_censo = "cod_depto")

  }

    if(geo == "ARGENTINA") {

    df


  } else {

    ##############  PROVINCES MAPS ######################


    temp <- geoAr::show_arg_codes(viewer = FALSE) %>%
      dplyr::filter(id == geo) %>%
      dplyr::pull(codprov_censo)


    df <-   df %>%
      dplyr::filter(codprov_censo %in% temp)

  }

  # download message one per session hack
  if(base::getOption('descarga-bahra', TRUE)){


    message(glue::glue("Los datos fueron obtenidos del proyecto 'Base de Asentamientos Humanos de la Republica Argentina (BAHRA)'. La documentacion se encuetra disponible en http://www.bahra.gob.ar/"))

    options('ddescarga-bahra' = FALSE)




  return(df)



  }

}

#' Agrega columnas con id geográficos asignados a poligonos de provincias o departamentos de Argentina
#'   (\emph{Augment Argentina's districts polygons id})
#'@description
#'
#'Función que permite agregar columnas con ids geográficos para utilización de mapas de Argentina y de departamentos para los 24 distritos
#'  sub nacionales para una más sencilla la vinculación con bases de datos de diversas fuentes.
#'
#'@return Los valores por defecto en \code{\link{get_geo}} son \code{codprov_censo} para provincia y \code{coddepto_censo} para departamentos, respectivamente.
#' Estos corresponden a la codificación de INDEC.
#' Con el agregado usando \code{add_geo_codes} se puede optar por las nomenclaturas de \code{'indra'} - correspondiente a los escrutinios provisorios de elecciones nacionales,
#'  con las variantes \code{codprov} y \code{coddepto}, o la de \code{'iso'} con \code{\*_iso} , estas últimas hasta el nivel
#'  provincial.
#'
#'@details Respecto el origen de los datos se puede consultar la documentación de
#' \href{https://www.iso.org/obp/ui/#iso:code:3166:AR}{\code{ISO 3166-2} - International Organization for Standardization} y
#' del \href{https://www.iso.org/obp/ui/#iso:code:3166:AR}{\emph{INDEC}}.
#'
#'@details \code{codprov} y \code{coddepto} son las codificaciones de las bases de datos de \emph{INDRA}, empresa encargada por
#' muchos años de la tarea del escrutinio provisorio y utilizados en \href{https://electorarg.github.io/polAr/}{polAr}.
#
#'@param data data.frame obtenido con \code{\link{get_geo}}. Los mapas disponibles
#' se pueden chequear con \code{\link{show_arg_codes}}.
#'
#'
#'
#' @examples
#'
#' get_geo("TUCUMAN")
#'
#'
#' get_geo("TUCUMAN") %>%
#'    add_geo_codes()
#'
#'
#'@export

add_geo_codes <- function(data){


  # Check parameters

  assertthat::assert_that(is.data.frame(data),
                          msg = glue::glue("{data} debe ser un 'data.frame' obtenido con la funcion get_geo()"))

  assertthat::assert_that('geometry' %in% names(data),
                          msg = glue::glue("{data} debe ser un 'data.frame' obtenido con la funcion get_geo()"))




  level <-  base::names(data)


  if("coddepto_censo" %in% level){


  data %>%
    dplyr::left_join(geo_metadata, by = c("codprov_censo", "coddepto_censo")) %>%
    dplyr::relocate(geometry, .after = dplyr::last_col())



  } else {


    data %>%
      dplyr::left_join(geo_metadata %>%
                         dplyr::select(dplyr::contains("codprov"), name_iso) %>%
                         dplyr::distinct(), by = c("codprov_censo")) %>%
      dplyr::relocate(geometry, .after = dplyr::last_col())




  }




}

#'Recodifica id de grillas asignados a  provincias o departamentos de Argentina
#'   (\emph{Recode Argentina's districs id in grids})
#'@description
#'
#'Función que permite re codificar etiquetas para utilización de grillas de Argentina y de departamentos para los 24 distritos
#'  sub nacionales. Ello permite hacer mas sencilla la vinculación con bases de datos de diversas fuentes.
#'
#'@return Los valores por defecto en \code{\link{get_grid}} son \code{codprov} para provincia y \code{coddepto} para departamentos, respectivamente.
#' Estos corresponden a la codificación de los escrutinios provisorios de elecciones nacionales y se etiquetaron como \code{'indra'} .
#' Se puede optar por la nomenclatura de \code{'indec'}, con la familia \code{\*_censo}, para ambos niveles, o la de \code{'iso'} con \code{\*_iso} ,
#' para el nivel provincial.
#'
#'@details Respecto el origen de los datos se puede consultar la documentación de
#' \href{https://www.iso.org/obp/ui/#iso:code:3166:AR}{\code{ISO 3166-2} - International Organization for Standardization} y
#' del \href{https://www.iso.org/obp/ui/#iso:code:3166:AR}{\emph{INDEC}}.
#'
#'@details \code{codprov} y \code{coddepto} son las codificaciones de las bases de datos de \emph{INDRA}, empresa encargada por
#' muchos años de la tarea del escrutinio provisorio y utilizados en \href{https://electorarg.github.io/polAr/}{polAr}.
#'
#'@param type la variante del código que se quiere definir para la grilla. Las opciones son \code{'indra'}, \code{'indec'} o \code{'iso'}.
#'
#'@param data data.frame obtenido con \code{\link{get_grid}}. Las grillas disponibles
#' se pueden chequear con \code{\link{show_arg_codes}}.
#'
#'
#'
#' @examples
#'
#' get_grid("ARGENTINA")
#'
#'
#' get_grid("ARGENTINA") %>%
#'    recode_grid(type = "iso")
#'
#'
#'@export


recode_grid<- function(data,
                       type = NULL){

  # Check parameters

  assertthat::assert_that(is.data.frame(data),
                          msg = glue::glue("{data} debe ser un 'data.frame' obtenido con la funcion get_grid()"))

  assertthat::assert_that(dim(data)[2] == 5,
                          msg = glue::glue("{data} debe ser un 'data.frame' obtenido con la funcion get_grid()"))

  assertthat::assert_that(is.character(type),
                          msg = "type debe ser del tipo 'character'")




  if("CABA" %in% data$name){

    assertthat::assert_that(is.character(type),
                            msg = "type debe ser del tipo 'character'")


    assertthat::assert_that(length(type) == 1,
                            msg = glue::glue("{type} no es una opcion valida para recodificar grillas de provincias.
                                             Debe elegir una opcion entre 'indra', 'indec' o 'iso'"))

    assertthat::assert_that(type %in% c("indra", "indec", "iso"),
                            msg = glue::glue("{type} no es una opcion valida ('indra', 'indec', 'iso')"))


    full_codes <- geo_metadata %>%
      dplyr::select(codprov, codprov_censo, codprov_iso) %>%
      dplyr::distinct()

    if(type == "indec"){

      data %>%
        dplyr::left_join(full_codes, by = c("code" = "codprov")) %>%
        dplyr::select(name, row, col, code = codprov_censo)


    } else if(type == "iso"){

      data %>%
        dplyr::left_join(full_codes, by = c("code" = "codprov")) %>%
        dplyr::select(name, row, col, code = codprov_iso)


    } else{

      data %>%
        dplyr::select(name, row, col, code)

    }

  } else {

    assertthat::assert_that(is.character(type),
                            msg = "type debe ser del tipo 'character'")


    assertthat::assert_that(length(type) == 1,
                            msg = glue::glue("{type} no es una opcion valida para recodificar grillas de departamentos.
                                                    Debe elegir una opcion entre 'indra'o 'indec'"))

    assertthat::assert_that(type %in% c("indra", "indec"),
                            msg = glue::glue("{type} no es una opcion valida para recodificar grillas de departamentos.
                                                    Debe elegir una opcion entre 'indra'o 'indec'"))


    #### hack para filtrar grilla de departamento que se quiere recodear

    # Me traigo todos los id de depto de todas las provincias y genero un id unico
    full_codes <- geo_metadata %>%
      dplyr::select(coddepto, nomdepto_censo,coddepto_censo, name_prov) %>%
      dplyr::mutate(nomdepto_censo = stringr::str_to_upper(nomdepto_censo),
                    id = paste0(coddepto,
                                stringr::str_remove_all(string = nomdepto_censo,
                                                        pattern = " "),
                                name_prov))


    # agrego a la base de grillas el mismo codigo de id de la metadata
    grillas_depto_id <- grillas_geofacet  %>%
      dplyr::bind_rows(.id = "name_prov") %>%
      tidyr::as_tibble() %>%
      dplyr::slice(25:dim(.)[1]) %>%
      dplyr::mutate(code = stringr::str_pad(code, 3, "left", 0),
                    id = paste0(code,
                                stringr::str_remove_all(string = name,
                                                        pattern = " "),
                                name_prov))

    # Creo filtro para seleccionar grilla correcta
    filtro_provincia <- data %>%
      dplyr::left_join(grillas_depto_id) %>%
      dplyr::left_join(full_codes)

    filtro_id <- filtro_provincia %>%
      dplyr::pull(id)

    # filtro la grilla de interes
    data <- grillas_depto_id %>%
      dplyr::filter(id %in% filtro_id) %>%
      dplyr::select(name, row, col, code)

    #######################################################################

    if(type == "indec"){

      data %>%
        dplyr::mutate(code = dplyr::case_when(
          code == filtro_provincia$coddepto ~ filtro_provincia$coddepto_censo
        )) %>%
        dplyr::select(name, row, col, code)


    } else{

      data

    }


  }
}

#' Base de datos de grillas
#'
#' Grillas de Argentina para usar con \code{facet_geo()}
#'
#'
#' @format Una lista que contiene 25 \emph{data.frames} con 5 variables:
#' \describe{
#'   \item{code_provincia}{identificación del distrito al que pertenece la grilla}
#'   \item{col}{LONGITUD relativa de los distritos de segundo nivel en la grilla asemejando la geografía}
#'   \item{row}{LATITUD relativa de los distritos de segundo nivel en la grilla asemejando la geografía}
#'   \item{code}{código de identificación de cada uno de los distritos que componen la grilla}
#'   \item{name}{nombre de cada uno de los distritos que componen la grilla}
#'
#'   ...
#' }
#'
"grillas_geofacet"

#'Agrega capa de Argentina para mapa interactivo
#'   (\emph{Load Argentina Tiles Template})
#'@description
#'Función que descarga capa base de Argentina para mapa interactivo creado con la libreria \code{leaflet}.
#'
#'@param data debe ser un objeto con class "leaflet" "htmlwidget".
#'@return Capa base para mapas interactivos
#'@details  El objetivo es agregar referencias de la base sobre la que se visualizan geometrías de Argentina. Se pueden agregar capas como polígonos descargandolos con \code{\link{get_geo}}.
#' Los geometrías disponibles se pueden chequear con \code{\link{show_arg_codes}}.

#' @examples
#'
#'  get_geo("TUCUMAN") %>%
#'  leaflet::leaflet() %>%
#'  leaflet::addPolygons() %>%
#'  addArgTiles()
#'
#'
#'@export


addArgTiles <- function(data){

  data %>%
    leaflet::addTiles(urlTemplate = "https://wms.ign.gob.ar/geoserver/gwc/service/tms/1.0.0/mapabase_gris@EPSG%3A3857@png/{z}/{x}/{-y}.png")

}

#*' Equivalencias de códigos de identificación geográfica
#'
#' Un archivo de datos que contiene identificadores geográficos para los departamentos de Argentina y sus provincias
#' (\emph{A data file containing geographic identifiers for the departments of Argentina and their provinces})
#'
#' @format Un tibble con 526 filas y 8 variables:
#' \describe{
#'   \item{codprov, coddepto}{identificadores utilizados por \emph{INDRA} para escrutinios provisorios de elecciones
#'   nacionales}
#'   \item{\emph{x_censo}}{identificadores de unidades geográficas del \emph{INDEC}}
#'   \item{\emph{x_iso}}{identificadores de \emph{International Organization for Standardization}}
#'
#'   ...
#'}
#'
"geo_metadata"

#'Carga poligonos geográficos de distritos de Argentina
#'   (\emph{Load Argentina's districts geometries})
#'@description
#'Función que descarga (\emph{geometry}) para graficar con mapas
#'
#'
#' @examples
#'
#' get_geo("TUCUMAN")
#'
#'@param geo un character con el nombre del district que se quiere descargar.
#'Se pueden chequear el id con \code{\link{show_arg_codes}}.
#'@param level parametro opcional para descargar geometrías a nivel 'departamento' o 'censal'
#'cuando se solicita mapa nacional \code{get_geo(geo = "ARGNTINA", level = "departamento")}.
#'@param simplified por defecto es TRUE y determina la descarga de una versión simplificada de las geometrias.
#'Con FALSE descarga la versión original de INDEC
#' @return tibble con capa geografica de Argentina o distrito seleccionado (a diferentes niveles de agregación) correspondientes al CENSO 2010 - INDEC
#'@export

get_geo <- function(geo = NULL,
                    level = "departamento",
                    simplified = TRUE) {

  ## Check for internet conection
  attempt::stop_if_not(.x = curl::has_internet(),
                       msg = "Internet access was not detected. Please check your connection //
No se detecto acceso a internet. Por favor chequear la conexion.")


  # Check parameters

  assertthat::assert_that(!is.null(geo),
                          msg = "debe ingresar un geo valido. Chequear opciones con 'show_arg_codes()")

  assertthat::assert_that(is.character(geo),
                          msg = "geo debe ser del tipo 'character'. Chequear opciones con 'show_arg_codes()")

  assertthat::assert_that(geo %in% c("ARGENTINA", "CABA", "CATAMARCA", "CHACO", "CHUBUT", "CORDOBA", "CORRIENTES",
                                     "ENTRE RIOS", "FORMOSA", "JUJUY", "LA PAMPA", "LA RIOJA", "MENDOZA", "MISIONES",
                                     "NEUQUEN","BUENOS AIRES", "RIO NEGRO", "SALTA", "SANTA CRUZ", "SANTA FE", "SANTIAGO DEL ESTERO",
                                     "SAN JUAN", "SAN LUIS", "TIERRA DEL FUEGO", "TUCUMAN"),
                          msg = "no es un geo valido. Chequearlos con show_arg_codes()")



  url <- if(level == "departamento" & simplified == TRUE){

    "https://github.com/politicaargentina/data_warehouse/raw/master/geoAr/data/localidades_simplified.geojson"

  } else if (level == "departamento" & simplified == FALSE){

    "https://github.com/politicaargentina/data_warehouse/raw/master/geoAr/data_raw/localidades.geojson"

  }  else if (level == "censal" & simplified == TRUE){

      "https://github.com/PoliticaArgentina/data_warehouse/raw/master/geoAr/data/radios_simplified.geojson"

  } else if (geo == "ARGENTINA" & level == "provincia" & simplified == TRUE){

    "https://github.com/politicaargentina/data_warehouse/raw/master/geoAr/data/provincias_simplified.geojson"

  } else if (geo == "ARGENTINA" & level == "provincia" & simplified == FALSE){

    "https://github.com/politicaargentina/data_warehouse/raw/master/geoAr/data_raw/provincias.geojson"

  } else if (level == "censal" & simplified == FALSE){

    link <- "https://github.com/PoliticaArgentina/data_warehouse/raw/master/geoAr/data_raw/raw_radios_censales.zip"

    # Download file from URL

    # Create temfiles

    temp <- base::tempfile()

    temp2 <- base::tempfile()

    # Download .zip

    utils::download.file(url = link,  temp, quiet = TRUE)


    # Unzip files

    utils::unzip(zipfile = temp, exdir = temp2)

    # Select geojson file path
    (url <- base::list.files(temp2, pattern = "geojson$", full.names=TRUE))

  }






  # Set default value for try()

  default <- NULL

  df <- base::suppressWarnings(base::try(default <- sf::read_sf(url), silent = TRUE))

  if(is.null(default)){

    df <- base::message("Fail to download data. Source is not available // La fuente de datos no esta disponible")

  } else {

    df <- df

  }


  if("link" %in% names(df)){ # CENSUS TRACT DATA

    df <- df %>%
      dplyr::mutate(codprov_censo  = stringr::str_sub(string = link,
                                                      start = 1, end = 2),
                    coddepto_censo = stringr::str_sub(string = link,
                                                      start = 3, end = 5),
                    fraccion_censal = stringr::str_sub(string = link,
                                                       start = 6, end = 7),
                    radio_censal = stringr::str_sub(string = link,
                                                    start = 8, end = 9))


      }else{

        df <- df

        }

  ############## ARG MAP###############


  if(geo == "ARGENTINA") {

    assertthat::assert_that(level %in% c("departamento", "provincia", "censal"),
                            msg = "National geography can be downloaded only at 'censal', 'departamento' or 'provincia' level" )

    df


  } else {

    ##############  PROVINCES MAPS ######################

    assertthat::assert_that(level  %in% c("departamento", "censal"),
                            msg = "Provincial geography can be downloaded only at 'departamento' or 'censal' level" )



    temp <- geoAr::show_arg_codes(viewer = FALSE) %>%
      dplyr::filter(id == geo) %>%
      dplyr::pull(codprov_censo)


    df <-   df %>%
      dplyr::filter(codprov_censo %in% temp)

  }

  df

}

#' Geometrías de CENSOS
#' (\emph{geo ID's dictoniary})
#'
#'@description
#' Función que devuelve un \emph{data.frame} con listado de las geometrías de los CENSOS disponible
#' @examples
#' show_available_censos()
#' @return tibble con información auxiliar para descarga de CENSOS históricos con  \code{\link{get_censo}}
#'
#' @export



show_available_censos <- function(){

  print(censos)



}

#' @importFrom httr2 request req_url_query req_error req_auth_bearer_token resp_body_json resp_status resp_status_desc resp_url req_method req_body_json resp_body_raw resp_body_string resp_has_body resp_content_type req_perform req_perform_parallel
#' @importFrom promises then
NULL

check_internet <- function(){
  attempt::stop_if_not(.x = curl::has_internet(),
                       msg = "No se detect\u00f3 acceso a internet. Por favor chequea tu conexi\u00f3n.")
}

#' Replace NULL Values with NA
#'
#' Recursively replaces NULL values with NA in a list or vector.
#' This function is useful for handling NULL values in API responses
#' before converting them to data frames.
#'
#' @param x A list, vector, or single value to process
#' @return The input with all NULL values replaced by NA
#' @keywords internal
replace_null_with_na <- function(x) {
  if (is.list(x)) {
    # If it's a list, apply the function recursively to each element
    lapply(x, replace_null_with_na)
  } else if (is.null(x)) {
    # If it's NULL, replace with NA
    NA
  } else {
    # If it's not a list and not NULL, return as is
    x
  }
}

base_url <- "https://apis.datos.gob.ar/georef/api/"

# Define a reusable error handler for httr2 requests
httr2_error_handler <- function(resp) {
  # Attempt to get more specific error details from the body if available and small
  error_body_text <- ""
  tryCatch({
    # Only try to read body if it's likely text and not too large
    if (httr2::resp_has_body(resp) && grepl("json|text|xml", httr2::resp_content_type(resp), ignore.case = TRUE)) {
        body_content <- httr2::resp_body_string(resp, encoding = "UTF-8")
        if (nchar(body_content) < 500) { # Limit body size in error to avoid overly long messages
            error_body_text <- paste0(" - API Response: ", body_content)
        }
    }
  }, error = function(e) {
    # Ignore errors during error body retrieval
  })

  stop(paste0("API request failed: ",
              httr2::resp_status_desc(resp), " (", httr2::resp_status(resp), "). ",
              "URL: ", httr2::resp_url(resp),
              error_body_text),
       call. = FALSE)
}

# API Limits for batch POST requests
GEOREFAR_API_MAX_QUERIES_PER_BATCH <- 1000
GEOREFAR_API_MAX_SUM_MAX_PARAM_PER_BATCH <- 5000

# Internal helper function to create batches of queries
create_query_batches <- function(all_queries,
                                 max_queries_per_batch = GEOREFAR_API_MAX_QUERIES_PER_BATCH,
                                 max_sum_of_a_param = GEOREFAR_API_MAX_SUM_MAX_PARAM_PER_BATCH,
                                 param_name_for_sum = NULL) {
  if (!is.list(all_queries) || length(all_queries) == 0) {
    return(list())
  }

  batches <- list()
  current_batch_queries <- list()
  current_query_count_in_batch <- 0
  current_sum_val_in_batch <- 0

  for (query_idx in seq_along(all_queries)) {
    query <- all_queries[[query_idx]]

    query_param_val <- 0
    # Calculate the value of the parameter to be summed (e.g., 'max') for the current query
    if (!is.null(param_name_for_sum) && !is.null(query[[param_name_for_sum]])) {
      val <- suppressWarnings(as.numeric(query[[param_name_for_sum]]))
      if (!is.na(val) && val > 0) {
        query_param_val <- val
      }
    }

    # Determine if this query can be added to the current batch
    can_add_to_current_batch <- TRUE

    # Check query count limit
    if ((current_query_count_in_batch + 1) > max_queries_per_batch) {
      can_add_to_current_batch <- FALSE
    }

    # Check sum of parameter limit (if applicable)
    if (can_add_to_current_batch && !is.null(max_sum_of_a_param) && !is.null(param_name_for_sum)) {
      if ((current_sum_val_in_batch + query_param_val) > max_sum_of_a_param && query_param_val > 0) {
        # This check is relevant only if the query itself contributes to the sum (param_value > 0).
        # A query with param_value = 0 (or param not present) doesn't affect the sum limit.
        can_add_to_current_batch <- FALSE
      }
    }

    # If current batch has queries AND this query cannot be added, finalize the current batch
    if (length(current_batch_queries) > 0 && !can_add_to_current_batch) {
      batches <- append(batches, list(current_batch_queries))
      # Reset for new batch
      current_batch_queries <- list()
      current_query_count_in_batch <- 0
      current_sum_val_in_batch <- 0
    }

    # Add query to current batch (which might be new or the existing one)
    current_batch_queries <- append(current_batch_queries, list(query))
    current_query_count_in_batch <- current_query_count_in_batch + 1
    if (!is.null(param_name_for_sum)) { # Only add to sum if param_name_for_sum is defined
      current_sum_val_in_batch <- current_sum_val_in_batch + query_param_val
    }
  }

  # Add the last batch if it contains any queries
  if (length(current_batch_queries) > 0) {
    batches <- append(batches, list(current_batch_queries))
  }

  return(batches)
}

# Internal helper to PREPARE a single POST batch httr2_request object
prepare_post_batch_request <- function(endpoint, single_batch_queries_list) {
  token <- Sys.getenv("GEOREFAR_TOKEN")
  url <- paste0(base_url, endpoint)

  body_data <- list()
  body_data[[endpoint]] <- single_batch_queries_list

  req <- httr2::request(url) |>
    httr2::req_method("POST") |>
    httr2::req_body_json(data = body_data, auto_unbox = TRUE) |>
    # httr2::req_error will convert responses with status != 200 to an R error,
    # which req_perform_parallel(on_error="return") will then return as that error object.
    # The httr2_error_handler provides a formatted message for such errors.
    httr2::req_error(is_error = ~ httr2::resp_status(.x) != 200, body = httr2_error_handler)

  if (!is.null(token) && token != "") {
    req <- req |> httr2::req_auth_bearer_token(token)
  }

  return(req) # Return the request object, not a promise
}

# Internal helper to process the httr2 response object of a single POST batch
process_single_post_response <- function(response_obj, endpoint, num_queries_in_this_batch) {
  # This function assumes response_obj is a successful httr2_response
  parsed_content <- httr2::resp_body_json(response_obj, flatten = TRUE)

  if (!"resultados" %in% names(parsed_content)) {
    url_info <- tryCatch(httr2::resp_url(response_obj), error = function(e) "unknown URL")
    stop(paste0("La respuesta del POST para el endpoint '", endpoint, "' (URL: ", url_info, ") no contiene el campo 'resultados' esperado."), call. = FALSE)
  }

  results_list_from_api <- parsed_content$resultados

  # The actual data items are expected to be in results_list_from_api[[endpoint]]
  actual_data_items_list <- lapply(results_list_from_api, function(x) x[[endpoint]])

  if (is.null(actual_data_items_list)) {
    warning(paste0("Expected data field '", endpoint, "' not found or is NULL within the 'resultados' field of the bulk API response. Available keys in 'resultados': ", paste(names(results_list_from_api), collapse=", ")), call. = FALSE)
    return(dplyr::tibble())
  }

  if (!is.list(actual_data_items_list)) {
      warning(paste0("Data for endpoint '", endpoint, "' within 'resultados' is not a list as expected. Type: ", class(actual_data_items_list)), call. = FALSE)
      return(dplyr::tibble())
  }

  processed_results <- purrr::map_dfr(actual_data_items_list, function(data_items_for_one_original_query) {


    if (is.null(data_items_for_one_original_query)) {
      return(dplyr::tibble())
    }
    if (is.data.frame(data_items_for_one_original_query)) {
      return(dplyr::as_tibble(data_items_for_one_original_query))
    } else if (is.list(data_items_for_one_original_query)) {
      if (length(data_items_for_one_original_query) == 0) {
        return(dplyr::tibble())
      }

      tryCatch({
        # Ensure that elements being bound are suitable for bind_rows (e.g., named lists or data.frames)
        # If data_items_for_one_original_query is a list of atomic vectors or unnamed lists, this might fail.
        # Assuming API returns list of objects (named lists) or list of data.frames.
        data_items_for_one_original_query <- replace_null_with_na(data_items_for_one_original_query)

        return(dplyr::bind_rows(lapply(data_items_for_one_original_query, as.data.frame)))

      }, error = function(e) {
        warning(paste0("Failed to bind rows for an item in '", endpoint, "' results. Item class: ", class(data_items_for_one_original_query), ". Error: ", e$message), call. = FALSE)
        return(dplyr::tibble())
      })
    } else {
      warning(paste0("Unexpected data type ('", class(data_items_for_one_original_query), "') for an item in '", endpoint, "' results. Skipping this item."), call. = FALSE)
      return(dplyr::tibble())
    }
  })


  if (ncol(processed_results) > 0) {
    # processed_results <- processed_results |>
    #   dplyr::rename_with(.fn = function(x) {gsub(pattern = "\\\\$|\\\\.", replacement = "_", x = x)})
  }

  if (nrow(processed_results) == 0 && num_queries_in_this_batch > 0) {
    # This warning applies to a single batch. The overall warning will be in the calling post_*_bulk function.
    warning(paste0("Una tanda de ", num_queries_in_this_batch, " consultas POST para '", endpoint, "' devolvi\\u00f3 una lista vac\\u00eda o no se pudieron procesar sus resultados."), call. = FALSE)
  }

  return(processed_results)
}

get_endpoint <- function(endpoint, args) {

  args_clean <- purrr::discard(args, is.null)

  if (!assertthat::noNA(args_clean)) {
    stop(c('GET no admite NAs. Los par\u00e1metros siguientes tienen NAs:', sapply(names(args_clean[is.na(args_clean)]),
                                                                              function(x) paste0(" ", x),
                                                                              USE.NAMES = F)))
  }

  token <- Sys.getenv("GEOREFAR_TOKEN")

  req <- httr2::request(paste0(base_url, endpoint)) |>
    httr2::req_url_query(!!!args_clean) |>
    httr2::req_error(is_error = ~ httr2::resp_status(.x) != 200, body = httr2_error_handler)

  if (!is.null(token) && token != "") {
    req <- req |> httr2::req_auth_bearer_token(token)
  }

  # Use req_perform() for synchronous behavior needed by get_endpoint
  response <- httr2::req_perform(req)

  parsed <- httr2::resp_body_json(response)

  data_list <- parsed[[gsub(pattern = "-", replacement = "_", x = endpoint)]]

  if (is.null(data_list)) {
    data_list <- list() # Ensure it's an empty list if the key wasn't found or was null
  }

  data <- data_list |>
    purrr::modify_if(is.null, list) # Convert NULL elements within the list to list() for as_tibble

  if (length(data) == 0) {
    warning("La consulta devolvi\u00f3 una lista vac\u00eda", call. = FALSE)
  }

  data |>
    dplyr::bind_rows() |>
    dplyr::as_tibble(.name_repair = function(x) {gsub(pattern = "\\$|\\.", replacement = "_", x = x)})
}




#' Obtener Calles
#'
#' Permite realizar búsquedas sobre el listado de vías de circulación.
#' Realiza la consulta GET al endpoint /calles de georef-ar-api.
#' Si existe GEOREFAR_TOKEN en el Renviron lo usará para hacer la consulta.
#' @param id text Filtrar por ID.
#' @param nombre text Filtrar por Nombre.
#' @param tipo text Tipo de calle. (Valores disponibles: calle, avenida, pasaje.)
#' @param provincia text Filtrar por nombre o ID de provincia.
#' @param departamento text Filtrar por nombre o ID de departamento.
#' @param municipio text Filtrar por nombre o ID de municipio.
#' @param localidad_censal text Filtrar por nombre o ID de localidad censal.
#' @param categoria text Filtrar por categoría de calle.
#' @param interseccion text Geometría GeoJSON utilizada para filtrar resultados por intersección espacial. Sólo se soportan polígonos y multipolígonos. Ejemplo: polygon((-58.431,-34.592),(-58.430,-34.590),(-58.428,-34.593),(-58.431,-34.592)).
#' @param orden text Campo por el cual ordenar los resultados. (Por ID o nombre)
#' @param aplanar boolean Cuando está presente, muestra el resultado JSON con una estructura plana.
#' @param campos text Campos a incluir en la respuesta separados por comas, sin espacios. Algunos campos siempre serán incluidos, incluso si no se agregaron en la lista. Para incluir campos de sub-entidades, separar los nombres con un punto, por ejemplo: provincia.id.
#' @param max integer Cantidad máxima de resultados a devolver. Debe ser menor o igual a 5000.
#' @param inicio integer Cantidad de resultados a omitir desde el principio. La suma de 'max' e 'inicio' no debe superar 10000.
#' @param exacto boolean Cuando está presente, se activa el modo de búsqueda por texto exacto. Sólo tiene efecto cuando se usan campos de búsqueda por texto (por ejemplo, nombre).
#'
#' @export
#' @rdname get_calles
#'
#' @references [georef-ar-api/calles](https://datosgobar.github.io/georef-ar-api/open-api/#/Recursos/get_calles)
#' @return Un Data Frame con el listado de Calles
#' @examples
#' \dontrun{
#' get_calles(nombre = "Corrientes", provincia = "CIUDAD AUTONOMA DE BUENOS AIRES", max = 10)
#' }

get_calles <- function(nombre = NULL, id = NULL, tipo = NULL, provincia = NULL, departamento = NULL, municipio = NULL, localidad_censal = NULL, categoria = NULL, interseccion = NULL, orden = NULL, max = NULL, inicio = NULL, aplanar = TRUE, campos = NULL, exacto = NULL){

  assertthat::assert_that(max <= 5000 || is.null(max), msg = "par\u00e1metro 'max' debe ser menor a 5000 o NULL")
  assertthat::assert_that(max + inicio <= 10000 || is.null(max) || is.null(inicio), msg = "los par\u00e1metros 'max' e 'inicio' deben sumar 10.000 o menos")

  args <- list(nombre = nombre, id = id, tipo = tipo, provincia = provincia, departamento = departamento, municipio = municipio, localidad_censal = localidad_censal, categoria = categoria, interseccion = interseccion, orden = orden, inicio = inicio, aplanar = aplanar, campos = campos, exacto = exacto, max = max)

  endpoint <- "calles"

  check_internet()

  get_endpoint(endpoint = endpoint, args = args)
}

#' Enviar Lote de Consultas de Calles (POST)
#'
#' Permite realizar múltiples búsquedas sobre el listado de vías de circulación en una sola llamada POST.
#' Realiza la consulta POST al endpoint /calles de georef-ar-api.
#'
#' @param queries_list Lista de listas. Cada lista interna debe contener los parámetros
#'        para una consulta de calle individual.
#'        Parámetros válidos por consulta: nombre, id, tipo, provincia, departamento, municipio, localidad_censal, categoria, max, inicio, aplanar, campos, exacto.
#' @return Un Data Frame (tibble) con los resultados combinados de todas las consultas.
#' @export
#' @rdname post_calles_bulk
#'
#' @references [georef-ar-api/calles POST](https://datosgobar.github.io/georef-ar-api/open-api/#/Recursos/post_calles)
#' @examples
#' \dontrun{
#' consultas_calles <- list(
#'   list(nombre = "SAN MARTIN", provincia = "BUENOS AIRES", max = 2),
#'   list(nombre = "SARMIENTO", provincia = "CORDOBA", max = 2)
#' )
#' resultados_calles <- post_calles_bulk(queries_list = consultas_calles)
#' print(resultados_calles)
#' }
post_calles_bulk <- function(queries_list) {
  if (!is.list(queries_list) || !all(sapply(queries_list, is.list))) {
    stop("'queries_list' debe ser una lista de listas.")
  }
  if (length(queries_list) == 0) {
    warning("'queries_list' est\u00e1 vac\u00eda, no se realizar\u00e1 ninguna consulta.")
    return(dplyr::tibble())
  }

  valid_params <- c("id", "nombre", "tipo", "provincia", "departamento", "municipio", "localidad_censal", "categoria", "interseccion", "orden", "aplanar", "campos", "max", "inicio", "exacto")
  for (i in seq_along(queries_list)) {
    query <- queries_list[[i]]
    if (!is.list(query)) {
      stop(paste0("Elemento ", i, " en 'queries_list' no es una lista. Cada consulta debe ser una lista."))
    }
    param_names <- names(query)
    invalid_params <- setdiff(param_names, valid_params)
    if (length(invalid_params) > 0) {
      stop(paste0("Consulta ", i, " en 'queries_list' para 'calles' contiene par\u00e1metro(s) no reconocido(s): ",
                     paste(invalid_params, collapse = ", "), ". ",
                     "Par\u00e1metros v\u00e1lidos son: ", paste(valid_params, collapse = ", "), ". Estos par\u00e1metros ser\u00e1n enviados a la API, pero podr\u00edan ser ignorados o causar un error si no son aceptados por el endpoint espec\u00edfico."))
    }
    current_max <- query$max
    current_inicio <- query$inicio
    if (!is.null(current_max) && (!is.numeric(current_max) || current_max < 0 || current_max > 5000)) {
      stop(paste0("En la consulta ", i, ", el par\u00e1metro 'max' debe ser un n\u00famero entre 0 y 5000."))
    }
    if (!is.null(current_inicio) && (!is.numeric(current_inicio) || current_inicio < 0)) {
        stop(paste0("En la consulta ", i, ", el par\u00e1metro 'inicio' debe ser un n\u00famero positivo."))
    }
    if (!is.null(current_max) && !is.null(current_inicio) && (current_max + current_inicio > 10000)) {
      stop(paste0("En la consulta ", i, ", la suma de 'max' e 'inicio' no debe superar 10,000."))
    }
  }

  endpoint <- "calles"
  check_internet()

  query_batches <- create_query_batches(queries_list, param_name_for_sum = "max")
  if (length(query_batches) == 0 && length(queries_list) > 0) {
      warning(paste0("No se pudieron crear lotes de consultas para '", endpoint, "', aunque la lista de consultas no estaba vac\u00eda."), call. = FALSE)
      return(dplyr::tibble())
  } else if (length(query_batches) == 0) {
      return(dplyr::tibble())
  }

  all_batch_requests <- list()
  for (batch_idx in seq_along(query_batches)) {
    current_batch <- query_batches[[batch_idx]]
    request_obj <- prepare_post_batch_request(endpoint = endpoint, single_batch_queries_list = current_batch)
    all_batch_requests <- append(all_batch_requests, list(request_obj))
  }

  responses_or_errors <- httr2::req_perform_parallel(all_batch_requests, on_error = "return")
  final_results_list_of_tibbles <- list()
  has_errors <- FALSE
  for (i in seq_along(responses_or_errors)) {
    item <- responses_or_errors[[i]]
    num_queries_in_this_batch <- length(query_batches[[i]])
    if (inherits(item, "httr2_response")) {
      processed_tibble <- process_single_post_response(response_obj = item, endpoint = endpoint, num_queries_in_this_batch = num_queries_in_this_batch)
      final_results_list_of_tibbles <- append(final_results_list_of_tibbles, list(processed_tibble))
    } else if (inherits(item, "error")) {
      has_errors <- TRUE
      warning(paste0("Error en el lote ", i, " para '", endpoint, "': ", conditionMessage(item)), call. = FALSE)
    } else {
      has_errors <- TRUE
      warning(paste0("Error desconocido o respuesta inesperada en el lote ", i, " para '", endpoint, "'. Clase del objeto: ", class(item)[1]), call. = FALSE)
    }
  }
  combined_results <- dplyr::bind_rows(final_results_list_of_tibbles)
  if (nrow(combined_results) == 0 && length(queries_list) > 0 && !has_errors) {
    warning(paste0("La consulta POST completa para '", endpoint, "' (", length(queries_list)," consultas originales en ", length(query_batches)," lotes) devolvi\u00f3 una lista vac\u00eda o no se pudieron procesar los resultados, aunque no se reportaron errores directos en los lotes."), call. = FALSE)
  } else if (nrow(combined_results) == 0 && length(queries_list) > 0 && has_errors) {
    warning(paste0("La consulta POST completa para '", endpoint, "' (", length(queries_list)," consultas originales en ", length(query_batches)," lotes) no produjo resultados y se encontraron errores en algunos lotes."), call. = FALSE)
  }
  return(combined_results)
}

#' Obtener Departamentos
#'
#' Permite realizar búsquedas sobre el listado de departamentos.
#' Realiza la consulta GET al endpoint /departamentos de georef-ar-api.
#' Si existe GEOREFAR_TOKEN en el Renviron lo usará para hacer la consulta.
#' @param id text Filtrar por ID.
#' @param nombre text Filtrar por Nombre.
#' @param provincia text Filtrar por nombre o ID de Provincia.
#' @param interseccion text Geometría GeoJSON utilizada para filtrar resultados por intersección espacial. Sólo se soportan polígonos y multipolígonos. Ejemplo: polygon((-58.431,-34.592),(-58.430,-34.590),(-58.428,-34.593),(-58.431,-34.592)).
#' @param orden text Campo por el cual ordenar los resultados.  (Por ID o nombre)
#' @param aplanar boolean Cuando está presente, muestra el resultado JSON con una estructura plana.
#' @param campos text Campos a incluir en la respuesta separados por comas, sin espacios. Algunos campos siempre serán incluidos, incluso si no se agregaron en la lista. Para incluir campos de sub-entidades, separar los nombres con un punto, por ejemplo: provincia.id.
#' @param max integer Cantidad máxima de resultados a devolver. La API limita a un máximo de 529 para este endpoint.
#' @param inicio integer Cantidad de resultados a omitir desde el principio.
#' @param exacto boolean Cuando está presente, se activa el modo de búsqueda por texto exacto. Sólo tiene efecto cuando se usan campos de búsqueda por texto (por ejemplo, nombre).
#'
#' @export
#' @rdname get_departamentos
#'
#' @references [georef-ar-api/departamentos](https://datosgobar.github.io/georef-ar-api/open-api/#/Recursos/get_departamentos)
#' @return Un Data Frame con el listado de Departamentos
#' @examples
#' \dontrun{
#' get_departamentos(provincia = "06", max = 5)
#' }

get_departamentos <- function(id = NULL, nombre = NULL, provincia = NULL, interseccion = NULL, orden = NULL, aplanar = TRUE, campos = NULL, max = NULL, inicio = NULL, exacto = NULL){
  args <- list(id = id, nombre = nombre, provincia = provincia, interseccion = interseccion, orden = orden, aplanar = aplanar, campos = campos, max = max, inicio = inicio, exacto = exacto)

  endpoint <- "departamentos"

  check_internet()

  get_endpoint(endpoint = endpoint, args = args)
}

#' Enviar Lote de Consultas de Departamentos (POST)
#'
#' Permite realizar múltiples búsquedas sobre el listado de departamentos en una sola llamada POST.
#' Realiza la consulta POST al endpoint /departamentos de georef-ar-api.
#'
#' @param queries_list Lista de listas. Cada lista interna debe contener los parámetros
#'        para una consulta de departamento individual (e.g., list(nombre = "Rosario"), list(provincia = "02")).
#'        Parámetros válidos por consulta: id, nombre, provincia, orden, aplanar, campos, max, exacto.
#' @return Un Data Frame (tibble) con los resultados combinados de todas las consultas.
#' @export
#' @rdname post_departamentos_bulk
#'
#' @references [georef-ar-api/departamentos POST](https://datosgobar.github.io/georef-ar-api/open-api/#/Recursos/post_departamentos)
#' @examples
#' \dontrun{
#' consultas_deptos <- list(
#'   list(provincia = "22", nombre = "Ledesma"),
#'   list(id = "14028")
#' )
#' resultados_deptos <- post_departamentos_bulk(queries_list = consultas_deptos)
#' print(resultados_deptos)
#' }
post_departamentos_bulk <- function(queries_list) {
  if (!is.list(queries_list) || !all(sapply(queries_list, is.list))) {
    stop("'queries_list' debe ser una lista de listas.")
  }
  if (length(queries_list) == 0) {
    warning("'queries_list' est\u00e1 vac\u00eda, no se realizar\u00e1 ninguna consulta.")
    return(dplyr::tibble())
  }

  valid_params <- c("id", "nombre", "provincia", "interseccion", "orden", "aplanar", "campos", "max", "inicio", "exacto")
  for (i in seq_along(queries_list)) {
    query <- queries_list[[i]]
    if (!is.list(query)) {
      stop(paste0("Elemento ", i, " en 'queries_list' no es una lista. Cada consulta debe ser una lista."))
    }
    param_names <- names(query)
    invalid_params <- setdiff(param_names, valid_params)
    if (length(invalid_params) > 0) {
      warning(paste0("Consulta ", i, " en 'queries_list' para 'departamentos' contiene par\u00e1metro(s) no reconocido(s): ",
                     paste(invalid_params, collapse = ", "), ". ",
                     "Par\u00e1metros v\u00e1lidos son: ", paste(valid_params, collapse = ", "), "."))
    }
  }
  endpoint <- "departamentos"
  check_internet()
  query_batches <- create_query_batches(queries_list, param_name_for_sum = "max")
  if (length(query_batches) == 0 && length(queries_list) > 0) {
      warning(paste0("No se pudieron crear lotes de consultas para '", endpoint, "', aunque la lista de consultas no estaba vac\u00eda."), call. = FALSE)
      return(dplyr::tibble())
  } else if (length(query_batches) == 0) {
      return(dplyr::tibble())
  }
  all_batch_requests <- list()
  for (batch_idx in seq_along(query_batches)) {
    current_batch <- query_batches[[batch_idx]]
    request_obj <- prepare_post_batch_request(endpoint = endpoint, single_batch_queries_list = current_batch)
    all_batch_requests <- append(all_batch_requests, list(request_obj))
  }
  responses_or_errors <- httr2::req_perform_parallel(all_batch_requests, on_error = "return")
  final_results_list_of_tibbles <- list()
  has_errors <- FALSE
  for (i in seq_along(responses_or_errors)) {
    item <- responses_or_errors[[i]]
    num_queries_in_this_batch <- length(query_batches[[i]])
    if (inherits(item, "httr2_response")) {
      processed_tibble <- process_single_post_response(response_obj = item, endpoint = endpoint, num_queries_in_this_batch = num_queries_in_this_batch)
      final_results_list_of_tibbles <- append(final_results_list_of_tibbles, list(processed_tibble))
    } else if (inherits(item, "error")) {
      has_errors <- TRUE
      warning(paste0("Error en el lote ", i, " para '", endpoint, "': ", conditionMessage(item)), call. = FALSE)
    } else {
      has_errors <- TRUE
      warning(paste0("Error desconocido o respuesta inesperada en el lote ", i, " para '", endpoint, "'. Clase del objeto: ", class(item)[1]), call. = FALSE)
    }
  }
  combined_results <- dplyr::bind_rows(final_results_list_of_tibbles)
  if (nrow(combined_results) == 0 && length(queries_list) > 0 && !has_errors) {
    warning(paste0("La consulta POST completa para '", endpoint, "' (", length(queries_list)," consultas originales en ", length(query_batches)," lotes) devolvi\u00f3 una lista vac\u00eda o no se pudieron procesar los resultados, aunque no se reportaron errores directos en los lotes."), call. = FALSE)
  } else if (nrow(combined_results) == 0 && length(queries_list) > 0 && has_errors) {
    warning(paste0("La consulta POST completa para '", endpoint, "' (", length(queries_list)," consultas originales en ", length(query_batches)," lotes) no produjo resultados y se encontraron errores en algunos lotes."), call. = FALSE)
  }
  return(combined_results)
}

#' Normalizacion de direcciones
#'
#' Permite normalizar una dirección utilizando el listado de vías de circulación.
#' Realiza la consulta GET al endpoint /direcciones de georef-ar-api.
#' Si existe GEOREFAR_TOKEN en el Renviron lo usará para hacer la consulta.
#' @param direccion text Requerido. Direccion a normalizar, debe contener altura separada por espacio. (Ej: Colon 127)
#' @param provincia text Filtrar por nombre o ID de provincia.
#' @param departamento text Filtrar por nombre o ID de departamento.
#' @param localidad_censal text Filtrar por nombre o ID de localidad censal.
#' @param localidad text Filtrar por nombre o ID de localidad.
#' @param orden text Campo por el cual ordenar los resultados. (Por ID o nombre)
#' @param aplanar boolean Cuando está presente, muestra el resultado JSON con una estructura plana.
#' @param campos text Campos a incluir en la respuesta separados por comas, sin espacios. Algunos campos siempre serán incluidos, incluso si no se agregaron en la lista. Para incluir campos de sub-entidades, separar los nombres con un punto, por ejemplo: provincia.id.
#' @param max integer Cantidad máxima de resultados a devolver. La API limita a un máximo de 10 para este endpoint.
#' @param inicio integer Cantidad de resultados a omitir desde el principio.
#' @param exacto boolean Cuando está presente, se activa el modo de búsqueda por texto exacto. Sólo tiene efecto cuando se usan campos de búsqueda por texto (por ejemplo, nombre).
#'
#' @export
#' @rdname normalizar_direccion
#'
#' @references [georef-ar-api/direcciones](https://datosgobar.github.io/georef-ar-api/open-api/#/Recursos/get_direcciones)
#' @return Un Data Frame con el listado normalizado de de direcciones
#' @examples
#' \dontrun{
#' normalizar_direccion(direccion = "Corrientes 1200, Rosario")
#' normalizar_direccion(direccion = "SAN MARTIN 100", provincia = "02", max = 5)
#' }

normalizar_direccion <- function(direccion, provincia = NULL, departamento = NULL, localidad_censal = NULL, localidad = NULL, orden = NULL, aplanar = TRUE, campos = NULL, max = NULL, inicio = NULL, exacto = NULL){
  args <- list(direccion = direccion, provincia = provincia, departamento = departamento, localidad_censal = localidad_censal, localidad = localidad, orden = orden, aplanar = aplanar, campos = campos, max = max, inicio = inicio, exacto = exacto)

  endpoint <- "direcciones"

  check_internet()

  get_endpoint(endpoint = endpoint, args = args)
}

#' Normalizar un Lote de Direcciones (POST)
#'
#' Permite normalizar múltiples direcciones en una sola llamada POST.
#' Realiza la consulta POST al endpoint /direcciones de georef-ar-api.
#'
#' @param queries_list Lista de listas. Cada lista interna debe contener los parámetros
#'        para una consulta de normalización de dirección individual.
#'        Parámetro requerido por consulta: 'direccion' (e.g., "AV SAN MARTIN 123").
#'        Otros parámetros válidos: tipo, provincia, departamento, aplanar, campos, max, exacto.
#' @return Un Data Frame (tibble) con los resultados combinados de todas las normalizaciones.
#' @export
#' @rdname post_direcciones_bulk
#'
#' @references [georef-ar-api/direcciones POST](https://datosgobar.github.io/georef-ar-api/open-api/#/Recursos/post_direcciones)
#' @examples
#' \dontrun{
#' consultas_dir <- list(
#'   list(direccion = "MAIPU 100", provincia = "BUENOS AIRES"),
#'   list(direccion = "SANTA FE 2000, ROSARIO")
#' )
#' resultados_dir <- post_direcciones_bulk(queries_list = consultas_dir)
#' print(resultados_dir)
#' }
post_direcciones_bulk <- function(queries_list) {
  if (!is.list(queries_list) || !all(sapply(queries_list, is.list))) {
    stop("'queries_list' debe ser una lista de listas.")
  }
  if (length(queries_list) == 0) {
    warning("'queries_list' est\u00e1 vac\u00eda, no se realizar\u00e1 ninguna consulta.")
    return(dplyr::tibble())
  }

  valid_params <- c("direccion", "provincia", "departamento", "localidad_censal", "localidad", "aplanar", "campos", "max", "inicio", "exacto")
  for (i in seq_along(queries_list)) {
    query <- queries_list[[i]]
    if (!is.list(query)) {
      stop(paste0("Elemento ", i, " en 'queries_list' no es una lista. Cada consulta debe ser una lista."))
    }
    if (!"direccion" %in% names(query)) {
      stop(paste0("Consulta ", i, " en 'queries_list' para 'direcciones' debe contener un campo 'direccion'."))
    }
    param_names <- names(query)
    invalid_params <- setdiff(param_names, valid_params)
    if (length(invalid_params) > 0) {
      warning(paste0("Consulta ", i, " en 'queries_list' para 'direcciones' contiene par\u00e1metro(s) no reconocido(s): ",
                     paste(invalid_params, collapse = ", "), ". ",
                     "Par\u00e1metros v\u00e1lidos son: ", paste(valid_params, collapse = ", "), "."))
    }
  }
  endpoint <- "direcciones"
  check_internet()
  query_batches <- create_query_batches(queries_list, param_name_for_sum = "max")
  if (length(query_batches) == 0 && length(queries_list) > 0) {
      warning(paste0("No se pudieron crear lotes de consultas para '", endpoint, "', aunque la lista de consultas no estaba vac\u00eda."), call. = FALSE)
      return(dplyr::tibble())
  } else if (length(query_batches) == 0) {
      return(dplyr::tibble())
  }
  all_batch_requests <- list()
  for (batch_idx in seq_along(query_batches)) {
    current_batch <- query_batches[[batch_idx]]
    request_obj <- prepare_post_batch_request(endpoint = endpoint, single_batch_queries_list = current_batch)
    all_batch_requests <- append(all_batch_requests, list(request_obj))
  }
  responses_or_errors <- httr2::req_perform_parallel(all_batch_requests, on_error = "return")
  final_results_list_of_tibbles <- list()
  has_errors <- FALSE
  for (i in seq_along(responses_or_errors)) {
    item <- responses_or_errors[[i]]
    num_queries_in_this_batch <- length(query_batches[[i]])
    if (inherits(item, "httr2_response")) {
      processed_tibble <- process_single_post_response(response_obj = item, endpoint = endpoint, num_queries_in_this_batch = num_queries_in_this_batch)
      final_results_list_of_tibbles <- append(final_results_list_of_tibbles, list(processed_tibble))
    } else if (inherits(item, "error")) {
      has_errors <- TRUE
      warning(paste0("Error en el lote ", i, " para '", endpoint, "': ", conditionMessage(item)), call. = FALSE)
    } else {
      has_errors <- TRUE
      warning(paste0("Error desconocido o respuesta inesperada en el lote ", i, " para '", endpoint, "'. Clase del objeto: ", class(item)[1]), call. = FALSE)
    }
  }
  combined_results <- dplyr::bind_rows(final_results_list_of_tibbles)
  if (nrow(combined_results) == 0 && length(queries_list) > 0 && !has_errors) {
    warning(paste0("La consulta POST completa para '", endpoint, "' (", length(queries_list)," consultas originales en ", length(query_batches)," lotes) devolvi\u00f3 una lista vac\u00eda o no se pudieron procesar los resultados, aunque no se reportaron errores directos en los lotes."), call. = FALSE)
  } else if (nrow(combined_results) == 0 && length(queries_list) > 0 && has_errors) {
    warning(paste0("La consulta POST completa para '", endpoint, "' (", length(queries_list)," consultas originales en ", length(query_batches)," lotes) no produjo resultados y se encontraron errores en algunos lotes."), call. = FALSE)
  }
  return(combined_results)
}

#' Obtener Localidades
#'
#' Permite realizar búsquedas sobre el listado de localidades.
#' Realiza la consulta GET al endpoint /localidades de georef-ar-api.
#' Si existe GEOREFAR_TOKEN en el Renviron lo usará para hacer la consulta.
#' @param id text Filtrar por ID.
#' @param nombre text Filtrar por Nombre.
#' @param provincia text Filtrar por nombre o ID de Provincia.
#' @param departamento text Filtrar por nombre o ID de Departamento.
#' @param municipio text Filtrar por nombre o ID de Municipio.
#' @param interseccion text Geometría GeoJSON utilizada para filtrar resultados por intersección espacial. Sólo se soportan polígonos y multipolígonos. Ejemplo: polygon((-58.431,-34.592),(-58.430,-34.590),(-58.428,-34.593),(-58.431,-34.592)).
#' @param orden text Campo por el cual ordenar los resultados (por ID o nombre)
#' @param aplanar boolean Cuando está presente, muestra el resultado JSON con una estructura plana.
#' @param campos text Campos a incluir en la respuesta separados por comas, sin espacios. Algunos campos siempre serán incluidos, incluso si no se agregaron en la lista. Para incluir campos de sub-entidades, separar los nombres con un punto, por ejemplo: provincia.id.
#' @param max integer Cantidad máxima de resultados a devolver. La API limita a un máximo de 2000 para este endpoint.
#' @param inicio integer Cantidad de resultados a omitir desde el principio.
#' @param exacto boolean Cuando está presente, se activa el modo de búsqueda por texto exacto. Sólo tiene efecto cuando se usan campos de búsqueda por texto (por ejemplo, nombre).
#'
#' @export
#' @rdname get_localidades
#'
#' @references [georef-ar-api/localidades](https://datosgobar.github.io/georef-ar-api/open-api/#/Recursos/get_localidades)
#' @return Un Data Frame con el listado de Localidades
#' @examples
#' \dontrun{
#' get_localidades(nombre = "PALERMO", provincia = "CIUDAD AUTONOMA DE BUENOS AIRES")
#' }

get_localidades <- function(id = NULL, nombre = NULL, provincia = NULL, departamento = NULL, municipio = NULL, interseccion = NULL, orden = NULL, aplanar = TRUE, campos = NULL, max = NULL, inicio = NULL, exacto = NULL){
  args <- list(id = id, nombre = nombre, provincia = provincia, departamento = departamento, municipio = municipio, interseccion = interseccion, orden = orden, aplanar = aplanar, campos = campos, max = max, inicio = inicio, exacto = exacto)

  endpoint <- "localidades"

  check_internet()

  get_endpoint(endpoint = endpoint, args = args)
}

#' Enviar Lote de Consultas de Localidades (POST)
#'
#' Permite realizar múltiples búsquedas sobre el listado de localidades en una sola llamada POST.
#' Realiza la consulta POST al endpoint /localidades de georef-ar-api.
#'
#' @param queries_list Lista de listas. Cada lista interna debe contener los parámetros
#'        para una consulta de localidad individual.
#'        Parámetros válidos por consulta: id, nombre, provincia, departamento, municipio, orden, aplanar, campos, max, exacto.
#' @return Un Data Frame (tibble) con los resultados combinados de todas las consultas.
#' @export
#' @rdname post_localidades_bulk
#'
#' @references [georef-ar-api/localidades POST](https://datosgobar.github.io/georef-ar-api/open-api/#/Recursos/post_localidades)
#' @examples
#' \dontrun{
#' consultas_loc <- list(
#'   list(provincia = "Tucuman", departamento = "Capital"),
#'   list(id = "22056140000")
#' )
#' resultados_loc <- post_localidades_bulk(queries_list = consultas_loc)
#' print(resultados_loc)
#' }
post_localidades_bulk <- function(queries_list) {
  if (!is.list(queries_list) || !all(sapply(queries_list, is.list))) {
    stop("'queries_list' debe ser una lista de listas.")
  }
  if (length(queries_list) == 0) {
    warning("'queries_list' est\u00e1 vac\u00eda, no se realizar\u00e1 ninguna consulta.")
    return(dplyr::tibble())
  }

  valid_params <- c("id", "nombre", "provincia", "departamento", "municipio", "interseccion", "orden", "aplanar", "campos", "max", "inicio", "exacto")
  for (i in seq_along(queries_list)) {
    query <- queries_list[[i]]
    if (!is.list(query)) {
      stop(paste0("Elemento ", i, " en 'queries_list' no es una lista. Cada consulta debe ser una lista."))
    }
    param_names <- names(query)
    invalid_params <- setdiff(param_names, valid_params)
    if (length(invalid_params) > 0) {
      warning(paste0("Consulta ", i, " en 'queries_list' para 'localidades' contiene par\u00e1metro(s) no reconocido(s): ",
                     paste(invalid_params, collapse = ", "), ". ",
                     "Par\u00e1metros v\u00e1lidos son: ", paste(valid_params, collapse = ", "), "."))
    }
  }
  endpoint <- "localidades"
  check_internet()
  query_batches <- create_query_batches(queries_list, param_name_for_sum = "max")
  if (length(query_batches) == 0 && length(queries_list) > 0) {
      warning(paste0("No se pudieron crear lotes de consultas para '", endpoint, "', aunque la lista de consultas no estaba vac\u00eda."), call. = FALSE)
      return(dplyr::tibble())
  } else if (length(query_batches) == 0) {
      return(dplyr::tibble())
  }
  all_batch_requests <- list()
  for (batch_idx in seq_along(query_batches)) {
    current_batch <- query_batches[[batch_idx]]
    request_obj <- prepare_post_batch_request(endpoint = endpoint, single_batch_queries_list = current_batch)
    all_batch_requests <- append(all_batch_requests, list(request_obj))
  }
  responses_or_errors <- httr2::req_perform_parallel(all_batch_requests, on_error = "return")
  final_results_list_of_tibbles <- list()
  has_errors <- FALSE
  for (i in seq_along(responses_or_errors)) {
    item <- responses_or_errors[[i]]
    num_queries_in_this_batch <- length(query_batches[[i]])
    if (inherits(item, "httr2_response")) {
      processed_tibble <- process_single_post_response(response_obj = item, endpoint = endpoint, num_queries_in_this_batch = num_queries_in_this_batch)
      final_results_list_of_tibbles <- append(final_results_list_of_tibbles, list(processed_tibble))
    } else if (inherits(item, "error")) {
      has_errors <- TRUE
      warning(paste0("Error en el lote ", i, " para '", endpoint, "': ", conditionMessage(item)), call. = FALSE)
    } else {
      has_errors <- TRUE
      warning(paste0("Error desconocido o respuesta inesperada en el lote ", i, " para '", endpoint, "'. Clase del objeto: ", class(item)[1]), call. = FALSE)
    }
  }
  combined_results <- dplyr::bind_rows(final_results_list_of_tibbles)
  if (nrow(combined_results) == 0 && length(queries_list) > 0 && !has_errors) {
    warning(paste0("La consulta POST completa para '", endpoint, "' (", length(queries_list)," consultas originales en ", length(query_batches)," lotes) devolvi\u00f3 una lista vac\u00eda o no se pudieron procesar los resultados, aunque no se reportaron errores directos en los lotes."), call. = FALSE)
  } else if (nrow(combined_results) == 0 && length(queries_list) > 0 && has_errors) {
    warning(paste0("La consulta POST completa para '", endpoint, "' (", length(queries_list)," consultas originales en ", length(query_batches)," lotes) no produjo resultados y se encontraron errores en algunos lotes."), call. = FALSE)
  }
  return(combined_results)
}

#' Obtener Municipios
#'
#' Permite realizar búsquedas sobre el listado de municipios.
#' Realiza la consulta GET al endpoint /municipios de georef-ar-api.
#' Si existe GEOREFAR_TOKEN en el Renviron lo usará para hacer la consulta.
#' @param id text Filtrar por ID.
#' @param nombre text Filtrar por Nombre.
#' @param provincia text Filtrar por nombre o ID de Provincia.
#' @param departamento text Filtrar por nombre o ID de Departamento.
#' @param interseccion text Geometría GeoJSON utilizada para filtrar resultados por intersección espacial. Sólo se soportan polígonos y multipolígonos. Ejemplo: polygon((-58.431,-34.592),(-58.430,-34.590),(-58.428,-34.593),(-58.431,-34.592)).
#' @param orden text Campo por el cual ordenar los resultados. (Por ID o nombre)
#' @param aplanar boolean Cuando está presente, muestra el resultado JSON con una estructura plana.
#' @param campos text Campos a incluir en la respuesta separados por comas, sin espacios. Algunos campos siempre serán incluidos, incluso si no se agregaron en la lista. Para incluir campos de sub-entidades, separar los nombres con un punto, por ejemplo: provincia.id.
#' @param max integer Cantidad máxima de resultados a devolver. La API limita a un máximo de 2000 para este endpoint.
#' @param inicio integer Cantidad de resultados a omitir desde el principio.
#' @param exacto boolean Cuando está presente, se activa el modo de búsqueda por texto exacto. Sólo tiene efecto cuando se usan campos de búsqueda por texto (por ejemplo, nombre).
#'
#' @export
#' @rdname get_municipios
#'
#' @references [georef-ar-api/municipios](https://datosgobar.github.io/georef-ar-api/open-api/#/Recursos/get_municipios)
#' @return Un Data Frame con el listado de Municipios
#' @examples
#' \dontrun{
#' get_municipios(provincia = "cordoba", max = 10)
#' }

get_municipios <- function(id = NULL, nombre = NULL, provincia = NULL, departamento = NULL, interseccion = NULL, orden = NULL, aplanar = TRUE, campos = NULL, max = NULL, inicio = NULL, exacto = NULL){
  args <- list(id = id, nombre = nombre, provincia = provincia, departamento = departamento, interseccion = interseccion, orden = orden, aplanar = aplanar, campos = campos, max = max, inicio = inicio, exacto = exacto)

  endpoint <- "municipios"

  check_internet()

  get_endpoint(endpoint = endpoint, args = args)
}

#' Enviar Lote de Consultas de Municipios (POST)
#'
#' Permite realizar múltiples búsquedas sobre el listado de municipios en una sola llamada POST.
#' Realiza la consulta POST al endpoint /municipios de georef-ar-api.
#'
#' @param queries_list Lista de listas. Cada lista interna debe contener los parámetros
#'        para una consulta de municipio individual.
#'        Parámetros válidos por consulta: id, nombre, provincia, departamento, orden, aplanar, campos, max, exacto.
#' @return Un Data Frame (tibble) con los resultados combinados de todas las consultas.
#' @export
#' @rdname post_municipios_bulk
#'
#' @references [georef-ar-api/municipios POST](https://datosgobar.github.io/georef-ar-api/open-api/#/Recursos/post_municipios)
#' @examples
#' \dontrun{
#' consultas_muni <- list(
#'   list(provincia = "06", nombre = "La Plata"),
#'   list(id = "540098")
#' )
#' resultados_muni <- post_municipios_bulk(queries_list = consultas_muni)
#' print(resultados_muni)
#' }
post_municipios_bulk <- function(queries_list) {
  if (!is.list(queries_list) || !all(sapply(queries_list, is.list))) {
    stop("'queries_list' debe ser una lista de listas.")
  }
  if (length(queries_list) == 0) {
    warning("'queries_list' est\u00e1 vac\u00eda, no se realizar\u00e1 ninguna consulta.")
    return(dplyr::tibble())
  }

  valid_params <- c("id", "nombre", "provincia", "departamento", "interseccion", "orden", "aplanar", "campos", "max", "inicio", "exacto")
  for (i in seq_along(queries_list)) {
    query <- queries_list[[i]]
    if (!is.list(query)) {
      stop(paste0("Elemento ", i, " en 'queries_list' no es una lista. Cada consulta debe ser una lista."))
    }
    param_names <- names(query)
    invalid_params <- setdiff(param_names, valid_params)
    if (length(invalid_params) > 0) {
      warning(paste0("Consulta ", i, " en 'queries_list' para 'municipios' contiene par\u00e1metro(s) no reconocido(s): ",
                     paste(invalid_params, collapse = ", "), ". ",
                     "Par\u00e1metros v\u00e1lidos son: ", paste(valid_params, collapse = ", "), "."))
    }
  }
  endpoint <- "municipios"
  check_internet()
  query_batches <- create_query_batches(queries_list, param_name_for_sum = "max")
  if (length(query_batches) == 0 && length(queries_list) > 0) {
      warning(paste0("No se pudieron crear lotes de consultas para '", endpoint, "', aunque la lista de consultas no estaba vac\u00eda."), call. = FALSE)
      return(dplyr::tibble())
  } else if (length(query_batches) == 0) {
      return(dplyr::tibble())
  }
  all_batch_requests <- list()
  for (batch_idx in seq_along(query_batches)) {
    current_batch <- query_batches[[batch_idx]]
    request_obj <- prepare_post_batch_request(endpoint = endpoint, single_batch_queries_list = current_batch)
    all_batch_requests <- append(all_batch_requests, list(request_obj))
  }
  responses_or_errors <- httr2::req_perform_parallel(all_batch_requests, on_error = "return")
  final_results_list_of_tibbles <- list()
  has_errors <- FALSE
  for (i in seq_along(responses_or_errors)) {
    item <- responses_or_errors[[i]]
    num_queries_in_this_batch <- length(query_batches[[i]])
    if (inherits(item, "httr2_response")) {
      processed_tibble <- process_single_post_response(response_obj = item, endpoint = endpoint, num_queries_in_this_batch = num_queries_in_this_batch)
      final_results_list_of_tibbles <- append(final_results_list_of_tibbles, list(processed_tibble))
    } else if (inherits(item, "error")) {
      has_errors <- TRUE
      warning(paste0("Error en el lote ", i, " para '", endpoint, "': ", conditionMessage(item)), call. = FALSE)
    } else {
      has_errors <- TRUE
      warning(paste0("Error desconocido o respuesta inesperada en el lote ", i, " para '", endpoint, "'. Clase del objeto: ", class(item)[1]), call. = FALSE)
    }
  }
  combined_results <- dplyr::bind_rows(final_results_list_of_tibbles)
  if (nrow(combined_results) == 0 && length(queries_list) > 0 && !has_errors) {
    warning(paste0("La consulta POST completa para '", endpoint, "' (", length(queries_list)," consultas originales en ", length(query_batches)," lotes) devolvi\u00f3 una lista vac\u00eda o no se pudieron procesar los resultados, aunque no se reportaron errores directos en los lotes."), call. = FALSE)
  } else if (nrow(combined_results) == 0 && length(queries_list) > 0 && has_errors) {
    warning(paste0("La consulta POST completa para '", endpoint, "' (", length(queries_list)," consultas originales en ", length(query_batches)," lotes) no produjo resultados y se encontraron errores en algunos lotes."), call. = FALSE)
  }
  return(combined_results)
}

#' Obtener Provincias
#'
#' Permite realizar búsquedas sobre el listado de provincias.
#' Realiza la consulta GET al endpoint /provincias de georef-ar-api.
#' Si existe GEOREFAR_TOKEN en el Renviron lo usará para hacer la consulta.
#' @param id text Filtrar por ID.
#' @param nombre text Filtrar por Nombre.
#' @param interseccion text Geometría GeoJSON utilizada para filtrar resultados por intersección espacial. Sólo se soportan polígonos y multipolígonos. Ejemplo: polygon((-58.431,-34.592),(-58.430,-34.590),(-58.428,-34.593),(-58.431,-34.592)).
#' @param orden text Campo por el cual ordenar los resultados. (Por ID o nombre)
#' @param aplanar boolean Cuando está presente, muestra el resultado JSON con una estructura plana.
#' @param campos text Campos a incluir en la respuesta separados por comas, sin espacios. Algunos campos siempre serán incluidos, incluso si no se agregaron en la lista. Para incluir campos de sub-entidades, separar los nombres con un punto, por ejemplo: provincia.id.
#' @param max integer Cantidad máxima de resultados a devolver. La API limita a un máximo de 24 para este endpoint.
#' @param inicio integer Cantidad de resultados a omitir desde el principio.
#' @param exacto boolean Cuando está presente, se activa el modo de búsqueda por texto exacto. Sólo tiene efecto cuando se usan campos de búsqueda por texto (por ejemplo, nombre).
#'
#' @export
#' @rdname get_provincias
#'
#' @references [georef-ar-api/provincias](https://datosgobar.github.io/georef-ar-api/open-api/#/Recursos/get_provincias)
#' @return Un Data Frame con el listado de Provincias
#' @examples
#' \dontrun{
#' get_provincias(nombre = "Cordoba")
#' }

get_provincias <- function(id = NULL, nombre = NULL, interseccion = NULL, orden = NULL, aplanar = TRUE, campos = NULL, max = NULL, inicio = NULL, exacto = NULL){
  args <- list(id = id, nombre = nombre, interseccion = interseccion, orden = orden, aplanar = aplanar, campos = campos, max = max, inicio = inicio, exacto = exacto)

  endpoint <- "provincias"

  check_internet()

  get_endpoint(endpoint = endpoint, args = args)
}

#' Enviar Lote de Consultas de Provincias (POST)
#'
#' Permite realizar múltiples búsquedas sobre el listado de provincias en una sola llamada POST.
#' Realiza la consulta POST al endpoint /provincias de georef-ar-api.
#' Si existe GEOREFAR_TOKEN en el Renviron lo usará para hacer la consulta.
#'
#' @param queries_list Lista de listas. Cada lista interna debe contener los parámetros
#'        para una consulta de provincia individual (e.g., list(nombre = "Tucuman"), list(id = "06")).
#'        Parámetros válidos por consulta: id, nombre, orden, aplanar, campos, max, exacto.
#' @return Un Data Frame (tibble) con los resultados combinados de todas las consultas.
#'         Las respuestas de la API para cada consulta en el lote se apilan.
#' @export
#' @rdname post_provincias_bulk
#'
#' @references [georef-ar-api/provincias POST](https://datosgobar.github.io/georef-ar-api/open-api/#/Recursos/post_provincias)
#' @examples
#' \dontrun{
#' consultitas <- list(
#'   list(nombre = "santiago del estero"),
#'   list(id = "82", campos = "id,nombre,centroide.lat,centroide.lon"),
#'   list(nombre = "tierra del fuego", campos = "completo")
#' )
#' resultados_provincias <- post_provincias_bulk(queries_list = consultitas)
#' print(resultados_provincias)
#' }
post_provincias_bulk <- function(queries_list) {
  if (!is.list(queries_list) || !all(sapply(queries_list, is.list))) {
    stop("'queries_list' debe ser una lista de listas (cada elemento debe ser una lista representando una consulta).")
  }
  if (length(queries_list) == 0) {
    warning("'queries_list' est\u00e1 vac\u00eda, no se realizar\u00e1 ninguna consulta.")
    return(dplyr::tibble())
  }
  valid_params <- c("id", "nombre", "interseccion", "orden", "aplanar", "campos", "max", "inicio", "exacto")
  for (i in seq_along(queries_list)) {
    query <- queries_list[[i]]
    if (!is.list(query)) {
      stop(paste0("Elemento ", i, " en 'queries_list' no es una lista. Cada consulta debe ser una lista."))
    }
    param_names <- names(query)
    invalid_params <- setdiff(param_names, valid_params)
    if (length(invalid_params) > 0) {
      warning(paste0("Consulta ", i, " en 'queries_list' para 'provincias' contiene par\u00e1metro(s) no reconocido(s): ",
                     paste(invalid_params, collapse = ", "), ". ",
                     "Par\u00e1metros v\u00e1lidos son: ", paste(valid_params, collapse = ", "), "."))
    }
  }
  endpoint <- "provincias"
  check_internet()
  query_batches <- create_query_batches(queries_list, param_name_for_sum = "max")
  if (length(query_batches) == 0 && length(queries_list) > 0) {
      warning(paste0("No se pudieron crear lotes de consultas para '", endpoint, "', aunque la lista de consultas no estaba vac\u00eda."), call. = FALSE)
      return(dplyr::tibble())
  } else if (length(query_batches) == 0) {
      return(dplyr::tibble())
  }
  all_batch_requests <- list()
  for (batch_idx in seq_along(query_batches)) {
    current_batch <- query_batches[[batch_idx]]
    request_obj <- prepare_post_batch_request(endpoint = endpoint, single_batch_queries_list = current_batch)
    all_batch_requests <- append(all_batch_requests, list(request_obj))
  }
  responses_or_errors <- httr2::req_perform_parallel(all_batch_requests, on_error = "return")
  final_results_list_of_tibbles <- list()
  has_errors <- FALSE
  for (i in seq_along(responses_or_errors)) {
    item <- responses_or_errors[[i]]
    num_queries_in_this_batch <- length(query_batches[[i]])
    if (inherits(item, "httr2_response")) {
      processed_tibble <- process_single_post_response(response_obj = item, endpoint = endpoint, num_queries_in_this_batch = num_queries_in_this_batch)
      final_results_list_of_tibbles <- append(final_results_list_of_tibbles, list(processed_tibble))
    } else if (inherits(item, "error")) {
      has_errors <- TRUE
      warning(paste0("Error en el lote ", i, " para '", endpoint, "': ", conditionMessage(item)), call. = FALSE)
    } else {
      has_errors <- TRUE
      warning(paste0("Error desconocido o respuesta inesperada en el lote ", i, " para '", endpoint, "'. Clase del objeto: ", class(item)[1]), call. = FALSE)
    }
  }
  combined_results <- dplyr::bind_rows(final_results_list_of_tibbles)
  if (nrow(combined_results) == 0 && length(queries_list) > 0 && !has_errors) {
    warning(paste0("La consulta POST completa para '", endpoint, "' (", length(queries_list)," consultas originales en ", length(query_batches)," lotes) devolvi\u00f3 una lista vac\u00eda o no se pudieron procesar los resultados, aunque no se reportaron errores directos en los lotes."), call. = FALSE)
  } else if (nrow(combined_results) == 0 && length(queries_list) > 0 && has_errors) {
    warning(paste0("La consulta POST completa para '", endpoint, "' (", length(queries_list)," consultas originales en ", length(query_batches)," lotes) no produjo resultados y se encontraron errores en algunos lotes."), call. = FALSE)
  }
  return(combined_results)
}

#' Obtener Ubicacion
#'
#' Permite realizar una georreferenciación inversa para un punto, informando cuales unidades territoriales lo contienen.
#' Realiza la consulta GET al endpoint /ubicacion de georef-ar-api.
#' Si existe GEOREFAR_TOKEN en el Renviron lo usará para hacer la consulta.
#' @param lat numeric Latitud del punto, en forma de número real con grados decimales.
#' @param lon numeric Longitud del punto, en forma de número real con grados decimales.
#' @param aplanar boolean Cuando está presente, muestra el resultado JSON con una estructura plana.
#' @param campos text Campos a incluir en la respuesta separados por comas, sin espacios. Algunos campos siempre serán incluidos, incluso si no se agregaron en la lista. Para incluir campos de sub-entidades, separar los nombres con un punto, por ejemplo: provincia.id.
#'
#' @export
#' @rdname get_ubicacion
#'
#' @references [georef-ar-api/ubicacion](https://datosgobar.github.io/georef-ar-api/open-api/#/Recursos/get_ubicacion)
#' @return Un Data Frame con las unidades territoriales que contienen el punto.
#' @examples
#' \dontrun{
#' get_ubicacion()
#' }

get_ubicacion <- function(lat, lon, aplanar = TRUE, campos = NULL){
  args <- list(lat = lat, lon = lon, aplanar = aplanar, campos = campos)

  endpoint <- "ubicacion"

  check_internet()

  get_endpoint(endpoint = endpoint, args = args)
}

#' Georreferenciación Inversa para un Lote de Puntos (POST)
#'
#' Permite realizar georreferenciación inversa para múltiples puntos (lat, lon) en una sola llamada POST.
#' Realiza la consulta POST al endpoint /ubicacion de georef-ar-api.
#'
#' @param queries_list Lista de listas. Cada lista interna debe contener los parámetros
#'        'lat' y 'lon' para un punto.
#'        Otros parámetros válidos por consulta: aplanar, campos.
#' @return Un Data Frame (tibble) con los resultados combinados de todas las georreferenciaciones.
#' @export
#' @rdname post_ubicacion_bulk
#'
#' @references [georef-ar-api/ubicacion POST](https://datosgobar.github.io/georef-ar-api/open-api/#/Recursos/post_ubicacion)
#' @examples
#' \dontrun{
#' consultas_ubic <- list(
#'   list(lat = -34.6037, lon = -58.3816), # Buenos Aires
#'   list(lat = -32.9587, lon = -60.6393)  # Rosario
#' )
#' resultados_ubic <- post_ubicacion_bulk(queries_list = consultas_ubic)
#' print(resultados_ubic)
#' }
post_ubicacion_bulk <- function(queries_list) {
  if (!is.list(queries_list) || !all(sapply(queries_list, is.list))) {
    stop("'queries_list' debe ser una lista de listas.")
  }
  if (length(queries_list) == 0) {
    warning("'queries_list' est\u00e1 vac\u00eda, no se realizar\u00e1 ninguna consulta.")
    return(dplyr::tibble())
  }

  valid_params <- c("lat", "lon", "aplanar", "campos")
  for (i in seq_along(queries_list)) {
    query <- queries_list[[i]]
    if (!is.list(query)) {
      stop(paste0("Elemento ", i, " en 'queries_list' no es una lista. Cada consulta debe ser una lista."))
    }
    if (!all(c("lat", "lon") %in% names(query))) {
      stop(paste0("Consulta ", i, " en 'queries_list' para 'ubicacion' debe contener los campos 'lat' y 'lon'."))
    }
    param_names <- names(query)
    invalid_params <- setdiff(param_names, valid_params)
    if (length(invalid_params) > 0) {
      warning(paste0("Consulta ", i, " en 'queries_list' para 'ubicacion' contiene par\u00e1metro(s) no reconocido(s): ",
                     paste(invalid_params, collapse = ", "), ". ",
                     "Par\u00e1metros v\u00e1lidos son: ", paste(valid_params, collapse = ", "), "."))
    }
  }

  endpoint <- "ubicacion"
  check_internet()

  # 'ubicacion' endpoint does not use 'max', so sum limit is not applicable based on 'max'.
  # Batching will be based on query count only.
  query_batches <- create_query_batches(queries_list, param_name_for_sum = NULL)

  if (length(query_batches) == 0 && length(queries_list) > 0) {
      warning(paste0("No se pudieron crear lotes de consultas para '", endpoint, "', aunque la lista de consultas no estaba vac\u00eda."), call. = FALSE)
      return(dplyr::tibble())
  } else if (length(query_batches) == 0) {
      return(dplyr::tibble())
  }

  all_batch_requests <- list()
  for (batch_idx in seq_along(query_batches)) {
    current_batch <- query_batches[[batch_idx]]
    request_obj <- prepare_post_batch_request(endpoint = endpoint, single_batch_queries_list = current_batch)
    all_batch_requests <- append(all_batch_requests, list(request_obj))
  }

  responses_or_errors <- httr2::req_perform_parallel(all_batch_requests, on_error = "return")

  final_results_list_of_tibbles <- list()
  has_errors <- FALSE
  for (i in seq_along(responses_or_errors)) {
    item <- responses_or_errors[[i]]
    num_queries_in_this_batch <- length(query_batches[[i]])
    if (inherits(item, "httr2_response")) {
      processed_tibble <- process_single_post_response(response_obj = item, endpoint = endpoint, num_queries_in_this_batch = num_queries_in_this_batch)
      final_results_list_of_tibbles <- append(final_results_list_of_tibbles, list(processed_tibble))
    } else if (inherits(item, "error")) {
      has_errors <- TRUE
      warning(paste0("Error en el lote ", i, " para '", endpoint, "': ", conditionMessage(item)), call. = FALSE)
    } else {
      has_errors <- TRUE
      warning(paste0("Error desconocido o respuesta inesperada en el lote ", i, " para '", endpoint, "'. Clase del objeto: ", class(item)[1]), call. = FALSE)
    }
  }

  combined_results <- dplyr::bind_rows(final_results_list_of_tibbles)
  if (nrow(combined_results) == 0 && length(queries_list) > 0 && !has_errors) {
    warning(paste0("La consulta POST completa para '", endpoint, "' (", length(queries_list)," consultas originales en ", length(query_batches)," lotes) devolvi\u00f3 una lista vac\u00eda o no se pudieron procesar los resultados, aunque no se reportaron errores directos en los lotes."), call. = FALSE)
  } else if (nrow(combined_results) == 0 && length(queries_list) > 0 && has_errors) {
    warning(paste0("La consulta POST completa para '", endpoint, "' (", length(queries_list)," consultas originales en ", length(query_batches)," lotes) no produjo resultados y se encontraron errores en algunos lotes."), call. = FALSE)
  }
  return(combined_results)
}

#' Obtener Localidades Censales
#'
#' Permite realizar búsquedas sobre el listado de localidades censales.
#' Realiza la consulta GET al endpoint /localidades-censales de georef-ar-api.
#' Si existe GEOREFAR_TOKEN en el Renviron lo usará para hacer la consulta.
#' @param id text Filtrar por ID.
#' @param nombre text Filtrar por Nombre.
#' @param provincia text Filtrar por nombre o ID de Provincia.
#' @param departamento text Filtrar por nombre o ID de Departamento.
#' @param municipio text Filtrar por nombre o ID de Municipio.
#' @param interseccion text Geometría GeoJSON utilizada para filtrar resultados por intersección espacial. Sólo se soportan polígonos y multipolígonos. Ejemplo: polygon((-58.431,-34.592),(-58.430,-34.590),(-58.428,-34.593),(-58.431,-34.592)).
#' @param orden text Campo por el cual ordenar los resultados (por ID o nombre)
#' @param aplanar boolean Cuando está presente, muestra el resultado JSON con una estructura plana.
#' @param campos text Campos a incluir en la respuesta separados por comas, sin espacios. Algunos campos siempre serán incluidos, incluso si no se agregaron en la lista. Para incluir campos de sub-entidades, separar los nombres con un punto, por ejemplo: provincia.id.
#' @param max integer Cantidad máxima de resultados a devolver. La API limita a un máximo de 5000 para este endpoint.
#' @param inicio integer Cantidad de resultados a omitir desde el principio.
#' @param exacto boolean Cuando está presente, se activa el modo de búsqueda por texto exacto. Sólo tiene efecto cuando se usan campos de búsqueda por texto (por ejemplo, nombre).
#'
#' @export
#' @rdname get_localidades_censales
#'
#' @references [georef-ar-api/localidades-censales](https://datosgobar.github.io/georef-ar-api/open-api/#/Recursos/get_localidades_censales)
#' @return Un Data Frame con el listado de Localidades Censales.
#' @examples
#' \dontrun{
#' get_localidades_censales(nombre = "VILLA GENERAL BELGRANO")
#' }

get_localidades_censales <- function(id = NULL, nombre = NULL, provincia = NULL, departamento = NULL, municipio = NULL, interseccion = NULL, orden = NULL, aplanar = TRUE, campos = NULL, max = NULL, inicio = NULL, exacto = NULL){
  args <- list(id = id, nombre = nombre, provincia = provincia, departamento = departamento, municipio = municipio, interseccion = interseccion, orden = orden, aplanar = aplanar, campos = campos, max = max, inicio = inicio, exacto = exacto)

  endpoint <- "localidades-censales"

  check_internet()

  get_endpoint(endpoint = endpoint, args = args)
}

#' Enviar Lote de Consultas de Localidades Censales (POST)
#'
#' Permite realizar múltiples búsquedas sobre el listado de localidades censales en una sola llamada POST.
#' Realiza la consulta POST al endpoint /localidades-censales de georef-ar-api.
#'
#' @param queries_list Lista de listas. Cada lista interna debe contener los parámetros
#'        para una consulta de localidad censal individual.
#'        Parámetros válidos por consulta: id, nombre, provincia, departamento, municipio, orden, aplanar, campos, max, exacto.
#' @return Un Data Frame (tibble) con los resultados combinados de todas las consultas.
#' @export
#' @rdname post_localidades_censales_bulk
#'
#' @references [georef-ar-api/localidades-censales POST](https://datosgobar.github.io/georef-ar-api/open-api/#/Recursos/post_localidades_censales)
#' @examples
#' \dontrun{
#' consultas_loc_cen <- list(
#'   list(nombre = "AGUA DE ORO", provincia = "CÓRDOBA")
#' )
#' resultados_loc_cen <- post_localidades_censales_bulk(queries_list = consultas_loc_cen)
#' print(resultados_loc_cen)
#' }
post_localidades_censales_bulk <- function(queries_list) {
  if (!is.list(queries_list) || !all(sapply(queries_list, is.list))) {
    stop("'queries_list' debe ser una lista de listas.")
  }
  if (length(queries_list) == 0) {
    warning("'queries_list' est\u00e1 vac\u00eda, no se realizar\u00e1 ninguna consulta.")
    return(dplyr::tibble())
  }

  valid_params <- c("id", "nombre", "provincia", "departamento", "municipio", "interseccion", "orden", "aplanar", "campos", "max", "inicio", "exacto")
  for (i in seq_along(queries_list)) {
    query <- queries_list[[i]]
    if (!is.list(query)) {
      stop(paste0("Elemento ", i, " en 'queries_list' no es una lista. Cada consulta debe ser una lista."))
    }
    param_names <- names(query)
    invalid_params <- setdiff(param_names, valid_params)
    if (length(invalid_params) > 0) {
      warning(paste0("Consulta ", i, " en 'queries_list' para 'localidades-censales' contiene par\u00e1metro(s) no reconocido(s): ",
                     paste(invalid_params, collapse = ", "), ". ",
                     "Par\u00e1metros v\u00e1lidos son: ", paste(valid_params, collapse = ", "), "."))
    }
    # GET /localidades-censales has max=5000. POST might differ or use global batch limits.
  }

  endpoint <- "localidades-censales"
  check_internet()

  query_batches <- create_query_batches(queries_list, param_name_for_sum = "max")
  if (length(query_batches) == 0 && length(queries_list) > 0) {
      warning(paste0("No se pudieron crear lotes de consultas para '", endpoint, "', aunque la lista de consultas no estaba vac\u00eda."), call. = FALSE)
      return(dplyr::tibble())
  } else if (length(query_batches) == 0) {
      return(dplyr::tibble())
  }

  all_batch_requests <- list()
  for (batch_idx in seq_along(query_batches)) {
    current_batch <- query_batches[[batch_idx]]
    request_obj <- prepare_post_batch_request(endpoint = endpoint, single_batch_queries_list = current_batch)
    all_batch_requests <- append(all_batch_requests, list(request_obj))
  }

  responses_or_errors <- httr2::req_perform_parallel(all_batch_requests, on_error = "return")

  final_results_list_of_tibbles <- list()
  has_errors <- FALSE
  for (i in seq_along(responses_or_errors)) {
    item <- responses_or_errors[[i]]
    num_queries_in_this_batch <- length(query_batches[[i]])
    if (inherits(item, "httr2_response")) {
      processed_tibble <- process_single_post_response(response_obj = item, endpoint = endpoint, num_queries_in_this_batch = num_queries_in_this_batch)
      final_results_list_of_tibbles <- append(final_results_list_of_tibbles, list(processed_tibble))
    } else if (inherits(item, "error")) {
      has_errors <- TRUE
      warning(paste0("Error en el lote ", i, " para '", endpoint, "': ", conditionMessage(item)), call. = FALSE)
    } else {
      has_errors <- TRUE
      warning(paste0("Error desconocido o respuesta inesperada en el lote ", i, " para '", endpoint, "'. Clase del objeto: ", class(item)[1]), call. = FALSE)
    }
  }

  combined_results <- dplyr::bind_rows(final_results_list_of_tibbles)
  if (nrow(combined_results) == 0 && length(queries_list) > 0 && !has_errors) {
    warning(paste0("La consulta POST completa para '", endpoint, "' (", length(queries_list)," consultas originales en ", length(query_batches)," lotes) devolvi\u00f3 una lista vac\u00eda o no se pudieron procesar los resultados, aunque no se reportaron errores directos en los lotes."), call. = FALSE)
  } else if (nrow(combined_results) == 0 && length(queries_list) > 0 && has_errors) {
    warning(paste0("La consulta POST completa para '", endpoint, "' (", length(queries_list)," consultas originales en ", length(query_batches)," lotes) no produjo resultados y se encontraron errores en algunos lotes."), call. = FALSE)
  }
  return(combined_results)
}

#' Obtener Asentamientos de BAHRA
#'
#' Permite realizar búsquedas sobre el listado de asentamientos BAHRA.
#' Realiza la consulta GET al endpoint /asentamientos de georef-ar-api.
#' Si existe GEOREFAR_TOKEN en el Renviron lo usará para hacer la consulta.
#' @param id text Filtrar por ID.
#' @param nombre text Filtrar por Nombre.
#' @param provincia text Filtrar por nombre o ID de Provincia.
#' @param departamento text Filtrar por nombre o ID de Departamento.
#' @param municipio text Filtrar por nombre o ID de Municipio.
#' @param localidad_censal text Filtrar por nombre o ID de localidad censal. Se pueden especificar varios IDs separados por comas
#' @param interseccion text Geometría GeoJSON utilizada para filtrar resultados por intersección espacial. Sólo se soportan polígonos y multipolígonos. Ejemplo: polygon((-58.431,-34.592),(-58.430,-34.590),(-58.428,-34.593),(-58.431,-34.592)).
#' @param orden text Campo por el cual ordenar los resultados (por ID o nombre)
#' @param aplanar boolean Cuando está presente, muestra el resultado JSON con una estructura plana.
#' @param campos text Campos a incluir en la respuesta separados por comas, sin espacios. Algunos campos siempre serán incluidos, incluso si no se agregaron en la lista. Para incluir campos de sub-entidades, separar los nombres con un punto, por ejemplo: provincia.id.
#' @param max integer Cantidad máxima de resultados a devolver. La API limita a un máximo de 5000 para este endpoint.
#' @param inicio integer Cantidad de resultados a omitir desde el principio.
#' @param exacto boolean Cuando está presente, se activa el modo de búsqueda por texto exacto. Sólo tiene efecto cuando se usan campos de búsqueda por texto (por ejemplo, nombre).
#'
#' @export
#' @rdname get_asentamientos
#'
#' @references [georef-ar-api/asentamientos](https://datosgobar.github.io/georef-ar-api/open-api/#/Recursos/get_asentamientos)
#' @return Un Data Frame con el listado de Asentamientos BAHRA.
#' @examples
#' \dontrun{
#' get_asentamientos(provincia = "22", departamento = "007")
#' }

get_asentamientos <- function(id = NULL, nombre = NULL, provincia = NULL, departamento = NULL, municipio = NULL, localidad_censal = NULL, interseccion = NULL, orden = NULL, aplanar = TRUE, campos = NULL, max = NULL, inicio = NULL, exacto = NULL){
  args <- list(id = id, nombre = nombre, provincia = provincia, departamento = departamento, municipio = municipio, localidad_censal = localidad_censal, interseccion = interseccion, orden = orden, aplanar = aplanar, campos = campos, max = max, inicio = inicio, exacto = exacto)

  endpoint <- "asentamientos"

  check_internet()

  get_endpoint(endpoint = endpoint, args = args)
}

#' Enviar Lote de Consultas de Asentamientos (POST)
#'
#' Permite realizar múltiples búsquedas sobre el listado de asentamientos BAHRA en una sola llamada POST.
#' Realiza la consulta POST al endpoint /asentamientos de georef-ar-api.
#'
#' @param queries_list Lista de listas. Cada lista interna debe contener los parámetros
#'        para una consulta de asentamiento individual.
#'        Parámetros válidos por consulta: id, nombre, provincia, departamento, municipio, localidad_censal, orden, aplanar, campos, max, exacto.
#' @return Un Data Frame (tibble) con los resultados combinados de todas las consultas.
#' @export
#' @rdname post_asentamientos_bulk
#'
#' @references [georef-ar-api/asentamientos POST](https://datosgobar.github.io/georef-ar-api/open-api/#/Recursos/post_asentamientos)
#' @examples
#' \dontrun{
#' consultas_asent <- list(
#'   list(provincia = "CHUBUT", nombre = "COSTA")
#' )
#' resultados_asent <- post_asentamientos_bulk(queries_list = consultas_asent)
#' print(resultados_asent)
#' }
post_asentamientos_bulk <- function(queries_list) {
  if (!is.list(queries_list) || !all(sapply(queries_list, is.list))) {
    stop("'queries_list' debe ser una lista de listas.")
  }
  if (length(queries_list) == 0) {
    warning("'queries_list' est\u00e1 vac\u00eda, no se realizar\u00e1 ninguna consulta.")
    return(dplyr::tibble())
  }

  valid_params <- c("id", "nombre", "provincia", "departamento", "municipio", "localidad_censal", "interseccion", "orden", "aplanar", "campos", "max", "inicio", "exacto")
  for (i in seq_along(queries_list)) {
    query <- queries_list[[i]]
    if (!is.list(query)) {
      stop(paste0("Elemento ", i, " en 'queries_list' no es una lista. Cada consulta debe ser una lista."))
    }
    param_names <- names(query)
    invalid_params <- setdiff(param_names, valid_params)
    if (length(invalid_params) > 0) {
      warning(paste0("Consulta ", i, " en 'queries_list' para 'asentamientos' contiene par\u00e1metro(s) no reconocido(s): ",
                     paste(invalid_params, collapse = ", "), ". ",
                     "Par\u00e1metros v\u00e1lidos son: ", paste(valid_params, collapse = ", "), "."))
    }
    # GET /asentamientos has max=5000. POST might differ or use global batch limits.
  }

  endpoint <- "asentamientos"
  check_internet()

  query_batches <- create_query_batches(queries_list, param_name_for_sum = "max")
  if (length(query_batches) == 0 && length(queries_list) > 0) {
      warning(paste0("No se pudieron crear lotes de consultas para '", endpoint, "', aunque la lista de consultas no estaba vac\u00eda."), call. = FALSE)
      return(dplyr::tibble())
  } else if (length(query_batches) == 0) {
      return(dplyr::tibble())
  }

  # List to store httr2_request objects for each batch
  all_batch_requests <- list()
  for (batch_idx in seq_along(query_batches)) {
    current_batch <- query_batches[[batch_idx]]
    # Prepare the request object for the current batch
    request_obj <- prepare_post_batch_request(endpoint = endpoint,
                                              single_batch_queries_list = current_batch)
    all_batch_requests <- append(all_batch_requests, list(request_obj))
  }

  # Perform all batch requests in parallel
  # on_error = "return" ensures that errors are returned as objects in the list
  # instead of stopping the whole operation.
  responses_or_errors <- httr2::req_perform_parallel(all_batch_requests, on_error = "return")

  # Process results
  final_results_list_of_tibbles <- list()
  has_errors <- FALSE
  for (i in seq_along(responses_or_errors)) {
    item <- responses_or_errors[[i]]
    num_queries_in_this_batch <- length(query_batches[[i]])

    if (inherits(item, "httr2_response")) {
      # If req_error was set up correctly, non-200 status should have been converted to an R error object by httr2
      # So, if we get an httr2_response, it *should* be a successful one (status 200)
      # However, an extra check for status doesn't hurt, or rely on req_error having done its job.
      # For now, assume if it's an httr2_response, it's good to process based on req_error setup.
      processed_tibble <- process_single_post_response(response_obj = item,
                                                       endpoint = endpoint,
                                                       num_queries_in_this_batch = num_queries_in_this_batch)
      final_results_list_of_tibbles <- append(final_results_list_of_tibbles, list(processed_tibble))
    } else if (inherits(item, "error")) {
      # This is an error object, possibly from httr2_error_handler via req_error,
      # or a lower-level error from httr2/curl during the request that req_perform_parallel caught.
      has_errors <- TRUE
      warning(paste0("Error en el lote ", i, " para '", endpoint, "': ", conditionMessage(item)), call. = FALSE)
    } else {
      # Unexpected item type
      has_errors <- TRUE
      warning(paste0("Error desconocido o respuesta inesperada en el lote ", i, " para '", endpoint, "'. Clase del objeto: ", class(item)[1]), call. = FALSE)
    }
  }

  # Combine all tibbles from successfully processed batches
  combined_results <- dplyr::bind_rows(final_results_list_of_tibbles)

  if (nrow(combined_results) == 0 && length(queries_list) > 0 && !has_errors) {
    warning(paste0("La consulta POST completa para '", endpoint, "' (", length(queries_list)," consultas originales en ", length(query_batches)," lotes) devolvi\u00f3 una lista vac\u00eda o no se pudieron procesar los resultados, aunque no se reportaron errores directos en los lotes."), call. = FALSE)
  } else if (nrow(combined_results) == 0 && length(queries_list) > 0 && has_errors) {
    warning(paste0("La consulta POST completa para '", endpoint, "' (", length(queries_list)," consultas originales en ", length(query_batches)," lotes) no produjo resultados y se encontraron errores en algunos lotes."), call. = FALSE)
  }

  return(combined_results)
}

#' Descargar Datos Geográficos Completos
#'
#' Permite descargar listados completos de entidades geográficas en diversos formatos.
#' Accede al endpoint de la georef-ar-api.
#' Si existe GEOREFAR_TOKEN en el Renviron lo usará para hacer la consulta (aunque generalmente no es necesario para estos endpoints públicos).
#'
#' @param entidad Cadena de texto. La entidad geográfica a descargar.
#'        Valores posibles: "provincias", "departamentos", "municipios",
#'        "localidades", "localidades-censales", "asentamientos", "calles", "cuadras".
#' @param formato Cadena de texto. El formato deseado para el archivo.
#'        Valores posibles: "csv", "json", "geojson", "ndjson".
#' @param path_to_save Cadena de texto opcional. Ruta completa (incluyendo nombre de archivo y extensión)
#'        donde guardar el archivo descargado. Si es NULL (por defecto), la función devolverá el contenido
#'        parseado (para json/geojson/ndjson) o un data frame (para csv).
#'        Si se especifica una ruta, la función guardará el archivo y devolverá la ruta del archivo guardado.
#' @return Dependiendo de 'path_to_save' y 'formato':
#'         - Si 'path_to_save' se especifica: la ruta al archivo guardado (invisiblemente).
#'         - Si 'path_to_save' es NULL:
#'           - Para "csv": un data.frame.
#'           - Para "json", "geojson", "ndjson": una lista o estructura de R parseada desde JSON.
#'           - Si la descarga o parseo falla, genera un error.
#' @export
#' @rdname get_geodata_dump
#'
#' @references [georef-ar-api/descargas](https://datosgobar.github.io/georef-ar-api/open-api/#/Recursos/get__filename_)
#' @examples
#' \dontrun{
#'   # Obtener provincias en formato GeoJSON como objeto R
#'   provincias_geojson <- get_geodata_dump(entidad = "provincias", formato = "geojson")
#'
#'   # Guardar departamentos en formato CSV
#'   get_geodata_dump(entidad = "departamentos", formato = "csv", path_to_save = "deptos.csv")
#' }

get_geodata_dump <- function(entidad, formato, path_to_save = NULL) {

  check_internet()

  valid_entidades <- c("provincias", "departamentos", "municipios",
                       "localidades", "localidades-censales", "asentamientos",
                       "calles", "cuadras")
  valid_formatos <- c("csv", "json", "geojson", "ndjson")

  if (!entidad %in% valid_entidades) {
    stop(paste("Entidad no v\u00e1lida. Opciones disponibles:", paste(valid_entidades, collapse = ", ")))
  }
  if (!formato %in% valid_formatos) {
    stop(paste("Formato no v\u00e1lido. Opciones disponibles:", paste(valid_formatos, collapse = ", ")))
  }

  filename <- paste0(entidad, ".", formato)
  url <- paste0(base_url, filename)

  token <- Sys.getenv("GEOREFAR_TOKEN")

  req <- httr2::request(url) |>
    httr2::req_error(is_error = ~ httr2::resp_status(.x) != 200, body = httr2_error_handler)

  if (!is.null(token) && token != "") {
    req <- req |> httr2::req_auth_bearer_token(token)
  }

  # Use req_perform() for synchronous behavior
  response <- httr2::req_perform(req)

  if (!is.null(path_to_save)) {
    assertthat::assert_that(is.character(path_to_save), length(path_to_save) == 1,
                            msg = "'path_to_save' debe ser una cad\u00e1na de texto con la ruta del archivo.")
    tryCatch({
      writeBin(httr2::resp_body_raw(response), path_to_save)
      message(paste("Archivo guardado en:", path_to_save))
      return(invisible(path_to_save))
    }, error = function(e) {
      stop(paste("Error al guardar el archivo:", e$message))
    })
  } else {
    # Si no se guarda, intentar parsear según el formato
    # For JSON types, httr2::resp_body_json could be used directly.
    # For CSV, httr2::resp_body_string then read.csv is fine.
    # For consistency with original logic, use resp_body_string and then parse.
    content_text <- httr2::resp_body_string(response, encoding = "UTF-8")
    if (formato == "csv") {
      # httr2 can also directly parse CSVs with resp_body_csv(), but this is fine
      return(utils::read.csv(text = content_text, stringsAsFactors = FALSE))
    } else if (formato %in% c("json", "geojson", "ndjson")) {
      # httr2::resp_body_json(response, flatten = TRUE) could be used here too
      return(jsonlite::fromJSON(content_text, flatten = TRUE))
    } else {
      # This case should ideally not be reached due to prior format validation
      warning("Formato no soportado para parseo directo, devolviendo contenido como texto.", call. = FALSE)
      return(content_text)
    }
  }
}

#*' Listado de geometrías de CENSOS disponibles
#'
#' Un archivo de datos que contiene el listado de las geometrías para los censos disponibles
#'
#' @format Un tibble con 10 filas filas y 1 variables (censo)que funciona como id para usar con \code{\link{get_geo}}
#' \describe{
#'   \item{censo}{ID correspondiente al año del censo de Argentina para descargar poligonos disponibles (1869 - 2010)
#'
#'}}
#'
"censos"

#' Carga poligonos de Censos de Población Históricos de Argentina
#'
#' @param censo un character con el id del año del censo del district que se quiere descargar.
#'Se pueden chequear el id con \code{\link{show_available_censos}}.
#'
#' @param simplified por defecto es TRUE y determina la descarga de una versión simplificada de las geometrias.
#'Con FALSE descarga la versión original de INDEC
#' @return tibble con capa geografica correspondiente a geometrías de un censo histórico
#' @export
#'
#' @examples
#' get_censo(censo = "1947", simplified = TRUE)
#'
get_censo <- function(censo = NULL,
                      simplified = FALSE){

  ## Check for internet conection
  attempt::stop_if_not(.x = curl::has_internet(),
                       msg = "Internet access was not detected. Please check your connection //
No se detecto acceso a internet. Por favor chequear la conexion.")



  # Check parameters

  assertthat::assert_that(!is.null(censo),
                          msg = "debe ingresar un id de censo valido. Chequear opciones con 'show_available_censos()")

  assertthat::assert_that(is.character(censo),
                          msg = "geo debe ser del tipo 'character'. Chequear opciones con 'show_available_censos()")

  assertthat::assert_that(censo %in% censos$censo,
                          msg = "no es un id de censo valido. Chequearlos con show_available_censos")


  url <- if(simplified == TRUE){

    glue::glue("https://github.com/PoliticaArgentina/data_warehouse/raw/master/geoAr/data/censo{censo}_simplified.geojson")

  } else if (simplified == FALSE){

    glue::glue("https://github.com/PoliticaArgentina/data_warehouse/raw/master/geoAr/data_raw/censos/censo_{censo}.geojson")

  }

  # Set default value for try()

  default <- NULL

  df <- base::suppressWarnings(base::try(default <- sf::read_sf(url), silent = TRUE))

  if(is.null(default)){

    df <- base::message("Fail to download data. Source is not available // La fuente de datos no esta disponible")

  } else {

    df <- df


    # download message one per session hack
    if(base::getOption('descarga-censo', TRUE)){


      message(glue::glue("Los datos fueron obtenidos del proyecto 'Base cartografica de departamentos para los censos nacionales de la Republica Argentina desde 1869 a 2010' de Gonzalo Rodriguez y Pablo De Grande el {format(Sys.Date(), '%d %B de %Y')}. La documentacion se encuetra disponible en https://ri.conicet.gov.ar/handle/11336/149867"))

      options('descarga-censo' = FALSE)

    }


  }

  return(df)
}

#'Carga grillas de districts de Argentina
#'   (\emph{Load grids of districts of Argentina})
#'@description
#'Función que descarga grillas (\emph{facet}) para acomodarlas como si fueran mapas de provincias y deparamentos
#' de Argentina.
#'
#' @return tibble con datos correspondientes a diseño de grillas para utilizar con libreria \code{geofacet}
#' @examples
#'
#' get_grid("TUCUMAN")
#'
#'@param district un character con el nombre del district que se quiere descargar. Disponibles grillas para Argentina
#' y para las 24 provincias. Se pueden chequear los parametros con \code{\link{show_arg_codes}}.
#'@export


get_grid <- function(district = NULL) {


  # Check parameters

  assertthat::assert_that(!is.null(district),
                          msg = "debe ingresar un district valido. Chequear opciones con 'show_arg_codes()")

  assertthat::assert_that(is.character(district),
                          msg = "district debe ser del tipo 'character'. Chequear opciones con 'show_arg_codes()")

  assertthat::assert_that(district %in% c("ARGENTINA", "CABA", "CATAMARCA", "CHACO", "CHUBUT", "CORDOBA", "CORRIENTES",
                                          "ENTRE RIOS", "FORMOSA", "JUJUY", "LA PAMPA", "LA RIOJA", "MENDOZA", "MISIONES",
                                          "NEUQUEN","PBA", "RIO NEGRO", "SALTA", "SANTA CRUZ", "SANTA FE", "SANTIAGO DEL ESTERO",
                                          "SAN JUAN", "SAN LUIS", "TIERRA DEL FUEGO", "TUCUMAN", "AGLOMERADOS"),
                          msg = "no es un district valido. Chequearlos con show_arg_codes()")


  # Cargo geo-grids


  grillas <- grillas_geofacet %>%
    dplyr::bind_rows(.id = "name_provincia") %>%
    dplyr::group_by(name_provincia) %>%
    dplyr::select(row, col, code, name, name_provincia) %>%
    tidyr::nest()


  grillas %>%
    dplyr::filter(name_provincia == district) %>%
    tidyr::unnest(cols = c(data)) %>%
    dplyr::select(name, row, col, code) %>%
    as.data.frame()

}

#' \code{geoAr} package
#'
#' Caja de Herramientas GEO de Argentina
#' See the README on
#' \href{https://github.com/PoliticaArgentina/geoAr#readme}{Github}
#'
#' @docType package
#' @name geoAr
"_PACKAGE"

## quiets concerns of R CMD check re: the .'s that appear in pipelines

if(getRversion() >= "2.15.1")
  utils::globalVariables(c(".",
                           "df",
                           "coddept",
                           "coddepto",
                           "coddepto_censo",
                           "code",
                           "codprov",
                           "codprov_censo",
                           "codprov_iso",
                           "data",
                           "depto",
                           "geo_metadata",
                           "grillas_geofacet",
                           "id",
                           "name",
                           "name_iso",
                           "name_prov",
                           "name_provincia",
                           "nomdepto_censo",
                           "geometry",
                           "provincia",
                           "censos",
                           "cod_depto"))

#' Diccionario de códigos identificadores de distritos
#' (\emph{geo ID's dictoniary})
#'
#'@description
#' Función que devuelve un \emph{data.frame} con códigos y equivalencias de identificación de unidades geográficas
#'
#'@param viewer Por default es \code{TRUE} y  muestra una tabla formateada en el \emph{Viewer} de \emph{RStudio}. Cuando \code{FALSE} imprime en consola.
#'
#'@param nivel  Un character que permite elegir opción para ver diccionario a nivel de \code{"provincias"} o de \code{"departamentos"}.
#'
#' @return tibble con códigos geográficos correspondientes a distritos de Argentina descargados con \code{\link{get_geo}}
#' @examples
#'
#' show_arg_codes(viewer = FALSE)
#'
#' @export



show_arg_codes <- function(viewer = FALSE,
                           nivel  =  "provincias"){

  # Check parameters


  assertthat::assert_that(is.character(nivel),
                          msg = "nivel debe ser del tipo 'character'. Opciones 'provincias' (default) o 'departamentos'")

  assertthat::assert_that(nivel %in% c("provincias", "departamentos"),
                          msg = " 'provincias' o 'departamentos' son las unicas opciones validas")



  seleccion <-  if(nivel == "provincias"){

    geo_metadata %>%
      dplyr::select(id = name_prov, codprov, codprov_censo, codprov_iso, name_iso) %>%
      dplyr::distinct() %>%
      dplyr::add_row(.before = 1, id = "ARGENTINA", codprov = " ", codprov_censo = " ",
                     codprov_iso = "AR", name_iso = "Argentina")%>%
      dplyr::add_row(.after = 25, id = "AGLOMERADOS", codprov = " ", codprov_censo = " ",
                     codprov_iso = "AR", name_iso = "Aglomerados Urbanos - EPH (INDEC)")

  } else {

    geo_metadata %>%
      dplyr::select(id = name_prov, codprov, coddepto,
                    codprov_censo, coddepto_censo, nomdepto_censo) %>%
      dplyr::add_row(.before = 1, id = "ARGENTINA",
                     codprov = " ",
                     coddepto = " ",
                     codprov_censo = " ",
                     coddepto_censo = " ",
                     nomdepto_censo = " ")



  }



  if(viewer == FALSE){


    seleccion

  } else {


    selection <- dplyr::case_when(nivel == "provincias" ~ "Provincial",
                                  nivel == "departamentos" ~ "Departamental")

    seleccion %>%
      gt::gt()%>%
      gt::cols_label(id = gt::md(("ID grilla"))) %>%
      gt::tab_spanner(columns = dplyr::matches("censo"), label = "INDEC") %>%
      gt::tab_spanner(columns = dplyr::matches("iso"), label = "ISO") %>%
      gt::tab_spanner(columns = dplyr::matches("prov$|depto$"), label = "INDRA") %>%
      gt::tab_header(
        title = gt::md("**Diccionario de equivalencias para identificadores geo**"),
        subtitle = gt::md(glue::glue(("**Argentina - Nivel {selection}**")))) %>%
      gt::tab_source_note(
        source_note = gt::md("**Fuente:** geoAr  - *https://politicaargentina.github.io/geoAr/*"))



  }



}

#' Carga poligonos de los Aglomerados Urbanos correspondientes a la Encuesta Permanente de Hogares (INDEC)
#'
#' @param geo un character con el nombre del distrito que se quiere descargar.
#'Se puede chequear el id con \code{\link{show_arg_codes}}.
#'
#' @param simplified por defecto es TRUE y determina la descarga de una versión simplificada de las geometrias.
#'Con FALSE descarga la versión original de INDEC
#'
#' @param level por defecto devuelve a nivel `envolvente` pero puede descargarse a nivel `radios` y `entidades`
#'
#' @param centroid por defecto devuelve poligonos como geometry pero pueden descargarse puntos (centroides correspondientes al level especificado)
#' @return tibble con capa geografica correspondiente a alguna versión de geometrías utilizadas en la Encuesta Permanente de Hogares (EPH)
#' @export
#'
#' @examples
#' get_eph(geo = "TUCUMAN")
#'
get_eph <- function(geo = "ARGENTINA",
                    simplified = TRUE,
                    centroid = FALSE,
                    level = "envolventes"){

  ## Check for internet conection
  attempt::stop_if_not(.x = curl::has_internet(),
                       msg = "Internet access was not detected. Please check your connection //
No se detecto acceso a internet. Por favor chequear la conexion.")


  # Check parameters

  assertthat::assert_that(!is.null(geo),
                          msg = "debe ingresar un geo valido. Chequear opciones con 'show_arg_codes()")

  assertthat::assert_that(is.character(geo),
                          msg = "geo debe ser del tipo 'character'. Chequear opciones con 'show_arg_codes()")

  assertthat::assert_that(level %in% c("entidades", "radios", "envolventes"),
                          msg = "level debe ser una de las siguientes opciones: 'entidades', 'radios' o 'envolventes'")

  assertthat::assert_that(geo %in% c("ARGENTINA", "CABA", "CATAMARCA", "CHACO", "CHUBUT", "CORDOBA", "CORRIENTES",
                                     "ENTRE RIOS", "FORMOSA", "JUJUY", "LA PAMPA", "LA RIOJA", "MENDOZA", "MISIONES",
                                     "NEUQUEN","BUENOS AIRES", "RIO NEGRO", "SALTA", "SANTA CRUZ", "SANTA FE", "SANTIAGO DEL ESTERO",
                                     "SAN JUAN", "SAN LUIS", "TIERRA DEL FUEGO", "TUCUMAN"),
                          msg = "no es un geo valido. Chequearlos con show_arg_codes()")


          url <- if(simplified == TRUE){

            if(level == "entidades"){

              "https://github.com/PoliticaArgentina/data_warehouse/raw/master/geoAr/data/aglos_simplified.geojson"

            }else if(level == "radios"){

              "https://github.com/PoliticaArgentina/data_warehouse/raw/master/geoAr/data/radios_eph_simplified.geojson"

            }else if(level == "envolventes"){

              "https://github.com/PoliticaArgentina/data_warehouse/raw/master/geoAr/data/aglos_envolventes_simplified.geojson"
            }



          } else if (simplified == FALSE){

            if(level == "entidades"){

            "https://github.com/PoliticaArgentina/data_warehouse/raw/master/geoAr/data_raw/entidades_eph.geojson"

            }else if(level == "radios"){

            "https://github.com/PoliticaArgentina/data_warehouse/raw/master/geoAr/data_raw/radios_eph.geojson"

            }else if(level == "envolventes"){

            "https://github.com/PoliticaArgentina/data_warehouse/raw/master/geoAr/data_raw/aglos_envolventes.geojson"

            }
          }

  # Set default value for try()

  default <- NULL

  df <- base::suppressWarnings(base::try(default <- sf::read_sf(url), silent = TRUE))

  if(is.null(default)){

    df <- base::message("Fail to download data. Source is not available // La fuente de datos no esta disponible")

  } else {

    df <- df

  }

  ############## ARG MAP###############


  if(geo == "ARGENTINA") {

      df


  } else {

    ##############  PROVINCES MAPS ######################


    temp <- geoAr::show_arg_codes(viewer = FALSE) %>%
      dplyr::filter(id == geo) %>%
      dplyr::pull(codprov_censo)


    df <-   df %>%
      dplyr::filter(codprov %in% temp)

  }


  if(centroid == FALSE){


    df

  }else{


    base::suppressWarnings(sf::st_centroid(df))


  }




  }

---
title: "geoAr & geofacet"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{geoAr & geofacet}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---




```r
library(geoAr) # Argentina's Spatial Data Toolbox, [github::PoliticaArgentina/geoAr] v0.0.1.2
library(geofacet) # 'ggplot2' Faceting Utilities for Geographical Data, CRAN v0.2.0
#> The legacy packages maptools, rgdal, and rgeos, underpinning the sp package,
#> which was just loaded, will retire in October 2023.
#> Please refer to R-spatial evolution reports for details, especially
#> https://r-spatial.org/r/2023/05/15/evolution4.html.
#> It may be desirable to make the sf package available;
#> package maintainers should consider adding sf to Suggests:.
#> The sp package is now running under evolution status 2
#>      (status 2 uses the sf package in place of rgdal)
```

`{geoAr}` está pensado para facilitar el acceso a datos y el trabajo con información geográfica de argentina. Una de las posibilidades es la descarga de capas ge geometrías con el objetivo de visualizar información. Una alternativa típica es utilizar `{sf}` para trabajar con datos de este tipo y combinarlo con `{ggplot2}` (y su función `geom_sf()` para visualizar capas geográficas.)



```r

tucuman <- get_geo(geo = "TUCUMAN")

(tucuman_names <- tucuman %>%
  add_geo_codes()) # Augment data with metadata as districts names or alternative id codes
#> Simple feature collection with 17 features and 8 fields
#> Geometry type: MULTIPOLYGON
#> Dimension:     XY
#> Bounding box:  xmin: -66.18101 ymin: -28.01575 xmax: -64.48315 ymax: -26.06037
#> Geodetic CRS:  WGS 84
#> # A tibble: 17 × 9
#>    codprov_censo coddepto_censo codprov coddepto nomdepto_censo  name_prov codprov_iso
#>    <chr>         <chr>          <chr>   <chr>    <chr>           <chr>     <chr>      
#>  1 90            007            23      013      BURRUYACU       TUCUMAN   AR-T       
#>  2 90            014            23      012      CRUZ ALTA       TUCUMAN   AR-T       
#>  3 90            021            23      005      CHICLIGASTA     TUCUMAN   AR-T       
#>  4 90            028            23      003      FAMAILLA        TUCUMAN   AR-T       
#>  5 90            035            23      009      GRANEROS        TUCUMAN   AR-T       
#>  6 90            042            23      007      JUAN B. ALBERDI TUCUMAN   AR-T       
#>  7 90            049            23      008      LA COCHA        TUCUMAN   AR-T       
#>  8 90            056            23      011      LEALES          TUCUMAN   AR-T       
#>  9 90            063            23      002      LULES           TUCUMAN   AR-T       
#> 10 90            070            23      004      MONTEROS        TUCUMAN   AR-T       
#> 11 90            077            23      006      RIO CHICO       TUCUMAN   AR-T       
#> 12 90            084            23      001      CAPITAL         TUCUMAN   AR-T       
#> 13 90            091            23      010      SIMOCA          TUCUMAN   AR-T       
#> 14 90            098            23      017      TAFI DEL VALLE  TUCUMAN   AR-T       
#> 15 90            105            23      016      TAFI VIEJO      TUCUMAN   AR-T       
#> 16 90            112            23      014      TRANCAS         TUCUMAN   AR-T       
#> 17 90            119            23      015      YERBA BUENA     TUCUMAN   AR-T       
#> # ℹ 2 more variables: name_iso <chr>, geometry <MULTIPOLYGON [°]>


# MAPING

ggplot2::ggplot(data = tucuman_names) +
  ggplot2::geom_sf() +
  ggplot2::geom_sf_label(ggplot2::aes(label = nomdepto_censo))
#> Warning in st_point_on_surface.sfc(sf::st_zm(x)): st_point_on_surface may not give
#> correct results for longitude/latitude data
```

![](figure/tucmap-1.png)

No obstante, existen ocaciones en las que la visualización de información geográfica a través de mapas convencionales puede no ser la mejor opción. `{geofacet}` es un paquete que busca sacar provecho de la potencia de `{ggplot2}` y utilizar el _facetado_ de un modo que cada caja de una grilla represente una entidad y que la suma de estas guarden relación con su distribución espacial.


### Ejemplo

En los gráficos que siguen se representa un indicador tipico de la Ciencia Política (el _Número Efectivo de Partidos_) para una serie de tiempo para todas las provincias de Argentina. En el primer caso esto está facilitado al generar un _facet_ a partir de la variable PROVINCIA, generando un pequeño gráfico para cada distrito (_small multiple_).

![](https://tuqmanor.netlify.app/post/Geofacetear_files/figure-html/unnamed-chunk-1-1.png)


En el segundo se utiliza geofacet para acomodar esta grilla _cómo si fuera un mapa_ de Argentina. Facilitando de este modo una mirada que tenga en cuenta (y conocimiento) de la distribución geográfica de las provincias.


![](https://tuqmanor.netlify.app/post/Geofacetear_files/figure-html/unnamed-chunk-2-1.png)


>“El paquete geofacet amplía ggplot2 de una manera que facilita la creación de visualizaciones geográficamente facetadas en R. Para usar geofacet se toman datos que representan diferentes entidades geográficas y se aplica un método de visualización para cada entidad. El conjunto resultante de visualizaciones conforma una cuadrícula que imita la topología geográfica original”.

(Traducción propia de la nota intorudctoria del autor, Ryan Haffen. Fuente: https://tuqmano.ar/posts/Geofacetear.html)

Los siguientes puntos resumen las ventajas potenciales para el uso de esta herramientas en comparación de otras técnicas para representar información a través de mapas u otras alterantivas (como cartogramas, tilemaps, etc.):

* se pueden graficar múltiples variables para cada unidad geográfica

* se pueden aplicar otros esquemas de codificación visual más allá del color

* cada unidad geográfica tiene asignada la misma proporción espacial



Una particularidad del paquete `{geofacet}` es que propicia el desarrollo de grillas por parte de usuaries. Tanto para uso _ad hoc_ como para contribuir con un [repositorio de grillas que alimenta el paquete](https://hafen.github.io/geofacet/articles/geofacet.html#list-available-grids). Esta iniciativa nos impulsó a desarrollar grillas para Argentina y todas sus provincias, que terminaron conformando  `{geofacetAR}`, antecedente de este proyecto que tuvo sus primeros pasos en el marco del _1° LatinR_ en el que presentamos *Geofaceting Argentina*

[Abstract](https://github.com/TuQmano/geofacet_ARG/blob/master/.LatinR/Geofaceting_Argentina_RuizNicolini.pdf) | 📊 [Slides](https://www.researchgate.net/publication/327382101_Geofaceting_Argentina_LatinR_2018) | 📦 [{geofaceteAR}](https://electorarg.github.io/geofaceteAR/) | ⌨️ [blogpost](https://tuqmano.ar/posts/2020-05-22-empaquetar.html).



### get_grid()

Así como `get_geo()` permite descargar un set de datos que incluyen una variable `geometry` con información geográfica para la generación de mapas, `get_grid()` permite descargar grillas pre diseñadas para Argentina y cada una de las provincias.


```r

(tucuman_names <- tucuman %>%
  add_geo_codes()) # Augment data with metadata as districts names or alternative id codes
#> Simple feature collection with 17 features and 8 fields
#> Geometry type: MULTIPOLYGON
#> Dimension:     XY
#> Bounding box:  xmin: -66.18101 ymin: -28.01575 xmax: -64.48315 ymax: -26.06037
#> Geodetic CRS:  WGS 84
#> # A tibble: 17 × 9
#>    codprov_censo coddepto_censo codprov coddepto nomdepto_censo  name_prov codprov_iso
#>    <chr>         <chr>          <chr>   <chr>    <chr>           <chr>     <chr>      
#>  1 90            007            23      013      BURRUYACU       TUCUMAN   AR-T       
#>  2 90            014            23      012      CRUZ ALTA       TUCUMAN   AR-T       
#>  3 90            021            23      005      CHICLIGASTA     TUCUMAN   AR-T       
#>  4 90            028            23      003      FAMAILLA        TUCUMAN   AR-T       
#>  5 90            035            23      009      GRANEROS        TUCUMAN   AR-T       
#>  6 90            042            23      007      JUAN B. ALBERDI TUCUMAN   AR-T       
#>  7 90            049            23      008      LA COCHA        TUCUMAN   AR-T       
#>  8 90            056            23      011      LEALES          TUCUMAN   AR-T       
#>  9 90            063            23      002      LULES           TUCUMAN   AR-T       
#> 10 90            070            23      004      MONTEROS        TUCUMAN   AR-T       
#> 11 90            077            23      006      RIO CHICO       TUCUMAN   AR-T       
#> 12 90            084            23      001      CAPITAL         TUCUMAN   AR-T       
#> 13 90            091            23      010      SIMOCA          TUCUMAN   AR-T       
#> 14 90            098            23      017      TAFI DEL VALLE  TUCUMAN   AR-T       
#> 15 90            105            23      016      TAFI VIEJO      TUCUMAN   AR-T       
#> 16 90            112            23      014      TRANCAS         TUCUMAN   AR-T       
#> 17 90            119            23      015      YERBA BUENA     TUCUMAN   AR-T       
#> # ℹ 2 more variables: name_iso <chr>, geometry <MULTIPOLYGON [°]>


# GRIDS AS IF THEY WHERE MAPS FOR {geofacet}

(grid_tucuman <- get_grid("TUCUMAN"))
#> Adding missing grouping variables: `name_provincia`
#>    name_provincia            name row col code
#> 1         TUCUMAN       BURRUYACU   1   4  013
#> 2         TUCUMAN         CAPITAL   2   4  001
#> 3         TUCUMAN     CHICLIGASTA   4   2  005
#> 4         TUCUMAN       CRUZ ALTA   2   5  012
#> 5         TUCUMAN        FAMAILLA   3   3  003
#> 6         TUCUMAN        GRANEROS   5   4  009
#> 7         TUCUMAN JUAN B. ALBERDI   5   3  007
#> 8         TUCUMAN        LA COCHA   6   3  008
#> 9         TUCUMAN          LEALES   3   5  011
#> 10        TUCUMAN           LULES   3   4  002
#> 11        TUCUMAN        MONTEROS   3   2  004
#> 12        TUCUMAN       RIO CHICO   4   3  006
#> 13        TUCUMAN          SIMOCA   4   4  010
#> 14        TUCUMAN  TAFI DEL VALLE   2   1  017
#> 15        TUCUMAN      TAFI VIEJO   2   2  016
#> 16        TUCUMAN         TRANCAS   1   3  014
#> 17        TUCUMAN     YERBA BUENA   2   3  015

geofacet::grid_preview(grid_tucuman, label = "name")
#> Note: You provided a user-specified grid. If this is a generally-useful grid,
#>   please consider submitting it to become a part of the geofacet package. You
#>   can do this easily by calling: grid_submit(__grid_df_name__)
```

![](figure/tucgrid-1.png)

---
title: "geoAr & georeaf-ar"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{geoAr & georeaf-ar}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---




Dice la [documentación oficial](https://georef-ar-api.readthedocs.io/es/latest/) de la **API del Servicio de Normalización de Datos Geográficos de la Argentina** (`georef-ar`):

 > Permite normalizar y codificar los nombres de unidades territoriales de la Argentina (provincias, departamentos, municipios y localidades) y de sus calles, así como ubicar coordenadas dentro de ellas.

> En la [Guía para la identificación y uso de entidades interoperables](https://datosgobar.github.io/paquete-apertura-datos/guia-interoperables/) se explica cómo funcionan las [unidades territoriales internas de la Argentina](https://datosgobar.github.io/paquete-apertura-datos/guia-interoperables/#divisiones-o-unidades-territoriales-internas) y la relación entre ellas.

## Ejemplos mínimos de uso

#### NORMALIZAR (con `R`)

¿Cómo normalizar? Tomemos como ejemplo el siguiente listado de nombres de provincias _crudo_:


```r
library(geoAr) # Argentina's Spatial Data Toolbox, CRAN v0.1.4.2.1
library(dplyr) # A Grammar of Data Manipulation, CRAN v1.1.0
```



```r

normalizar_provincia <- tibble::tribble(
  ~provincia,
  "S. CRUZ",
  "Santiago",
  "TUCUMAN",
  "Sgo. del Estero",
  "mendoza",
  "CABA"
)
```


Veamos que sucede cuando pedimos a la API nos devuelva los datos de uno de los distritos de ese listado (CABA, el sexto elemento):


```r
  geoAr::get_provincias(nombre = normalizar_provincia$provincia[6], max = 1)
#> # A tibble: 1 × 4
#>   centroide_lat centroide_lon id    nombre                         
#>           <dbl>         <dbl> <chr> <chr>                          
#> 1         -34.6         -58.4 02    Ciudad Autónoma de Buenos Aires
```
La función `get_provincia` devuelve el nombre del distrito, el `id` y las coordenadas de latitud y longitud del centroide.

Podemos generar una función como la que sigue para luego iterar la consulta para todo el listado:



```r

normalizar <- function(base){

  tmp <- get_provincias(nombre = base, max = 1)

  tmp$nombre
}
```


Tomamos el `data.frame` original y generamos una nueva columna (`mutate`) dentro de la cual iteramos. Con ayuda de `map_chr` del paquete `purrr` pasamos la función antes creada al vector de nombres de provincias de nuestra base `normalizar_provincia`.


```r

(base_normalizada <- normalizar_provincia %>%
  dplyr::mutate(normalizada = purrr::map_chr(.x = provincia,
                                      .f = ~ normalizar(base = .x)
                                      )
         ))
#> # A tibble: 6 × 2
#>   provincia       normalizada                    
#>   <chr>           <chr>                          
#> 1 S. CRUZ         Santa Cruz                     
#> 2 Santiago        Santiago del Estero            
#> 3 TUCUMAN         Tucumán                        
#> 4 Sgo. del Estero Santiago del Estero            
#> 5 mendoza         Mendoza                        
#> 6 CABA            Ciudad Autónoma de Buenos Aires
```


### ENRIQUECER DATOS

Supongamos ahora que tenemos las coordenadas de lo que nos dicen es la ubicación del estadio del mejor equipo de fútbol del mundo, pero solo tenemos esa información:


```r
enriquecer_coordenadas <- tibble::tribble(
  ~latitud, ~longitud,
  -26.8129225,-65.1998249
)

leaflet::leaflet() %>%
  leaflet::addCircles(lng = enriquecer_coordenadas$longitud,
                      lat = enriquecer_coordenadas$latitud) %>%
  geoAr::addArgTiles()
#> Error in s$close(): attempt to apply non-function
```

Con `get_ubicacion` podemos agregarle información a nuestros datos de entrada:


```r

geoAr::get_ubicacion(lat = enriquecer_coordenadas$latitud,
                     lon = enriquecer_coordenadas$longitud)
#> # A tibble: 1 × 8
#>   departamento_id departamento_nombre   lat   lon municipio_id municipio_nombre     provincia_id provincia_nombre
#>   <chr>           <chr>               <dbl> <dbl> <chr>        <chr>                <chr>        <chr>           
#> 1 90084           Capital             -26.8 -65.2 900014       San Miguel de Tucum… 90           Tucumán
```


### GENERAR EL TOKEN

Los usuarios de organismos de la Administración Pública Nacional de Argentina pueden pedir un token para incrementar su cuota de uso de la API de Georef (ver <https://datosgobar.github.io/georef-ar-api/jwt-token/>).

Una vez que recibimos el secret y el key de la API de Georef, debemos generar el token y guardarlo en el Renviron. Las funciones de `geoAr` que consultan a la API de Georef van a buscar si existe el token en nuestro Renviron y van usarlo para realizar las consultas a la API.

Para generar el token a partir del secret y el key se puede usar el paquete `jose`

```
key <- "codigokey"
secreto <- "codigosecreto"

token <- jose::jwt_encode_hmac(claim = jose::jwt_claim(iss = key),
                               secret = secreto)

print(token)

```

El token generado debe ser guardado en el .Renviron con el siguiente formato:

```
GEOREFAR_TOKEN = "Xkxka1011skzlkz20201"

```

---
title: "geoAr & leaflet"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{geoAr & leaflet}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---







```r
library(geoAr) # Argentina's Spatial Data Toolbox, [github::PoliticaArgentina/geoAr] v0.0.1.2
library(leaflet) # Create Interactive Web Maps with the JavaScript 'Leaflet' Library, CRAN v2.0.4.1
```


`{geoAr}` facilita la descarga de polígonos para distintos niveles de agregación o división política de Argentina. Por ejemplo se puede descargar un mapa de todo el país al nivel de radios censales, departamentos o provincial. En el siguiente ejemplo se descargar un mapa nacional al nivel provincial y se utiliza sintaxis de `leaflet` para `R` para generar una visualización rápida interactiva.


```r

argentina <- get_geo(geo = "ARGENTINA", level = "provincia")

leaflet(argentina) %>%
  addPolygons()
#> PhantomJS not found. You can install it with webshot::install_phantomjs(). If it is installed, please make sure the phantomjs executable can be found via the PATH variable.
```

![](figure/map-1.png)

`leaflet` permite con una simple liena agregarle una capa base de una veriedad de proveedores distintos que permiten mayor contexto geográfico a la capa que se está representando. En el ejemplo siguiente se utiliza una capa de _Open Street Map_.


```r
leaflet(argentina) %>%
  addPolygons() %>%
  addProviderTiles(provider = providers$OpenStreetMap)
```

![](figure/mapOpenStreet-1.png)
`{goAr}` también incluye una función relativa a este punto. La misma se basa en la función anterior para permitir el agregado de una capa base desarrollada por el [Instituto Geográfico Nacional](https://www.ign.gob.ar/NuestrasActividades/InformacionGeoespacial/ServiciosOGC/Leaflet).


```r
leaflet(argentina) %>%
  addPolygons() %>%
  geoAr::addArgTiles()
```

![](figure/mapIGN-1.png)

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/georefar.R
\name{post_provincias_bulk}
\alias{post_provincias_bulk}
\title{Enviar Lote de Consultas de Provincias (POST)}
\usage{
post_provincias_bulk(queries_list)
}
\arguments{
\item{queries_list}{Lista de listas. Cada lista interna debe contener los parámetros
para una consulta de provincia individual (e.g., list(nombre = "Tucuman"), list(id = "06")).
Parámetros válidos por consulta: id, nombre, orden, aplanar, campos, max, exacto.}
}
\value{
Un Data Frame (tibble) con los resultados combinados de todas las consultas.
Las respuestas de la API para cada consulta en el lote se apilan.
}
\description{
Permite realizar múltiples búsquedas sobre el listado de provincias en una sola llamada POST.
Realiza la consulta POST al endpoint /provincias de georef-ar-api.
Si existe GEOREFAR_TOKEN en el Renviron lo usará para hacer la consulta.
}
\examples{
\dontrun{
consultitas <- list(
  list(nombre = "santiago del estero"),
  list(id = "82", campos = "id,nombre,centroide.lat,centroide.lon"),
  list(nombre = "tierra del fuego", campos = "completo")
)
resultados_provincias <- post_provincias_bulk(queries_list = consultitas)
print(resultados_provincias)
}
}
\references{
\href{https://datosgobar.github.io/georef-ar-api/open-api/#/Recursos/post_provincias}{georef-ar-api/provincias POST}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/georefar.R
\name{post_departamentos_bulk}
\alias{post_departamentos_bulk}
\title{Enviar Lote de Consultas de Departamentos (POST)}
\usage{
post_departamentos_bulk(queries_list)
}
\arguments{
\item{queries_list}{Lista de listas. Cada lista interna debe contener los parámetros
para una consulta de departamento individual (e.g., list(nombre = "Rosario"), list(provincia = "02")).
Parámetros válidos por consulta: id, nombre, provincia, orden, aplanar, campos, max, exacto.}
}
\value{
Un Data Frame (tibble) con los resultados combinados de todas las consultas.
}
\description{
Permite realizar múltiples búsquedas sobre el listado de departamentos en una sola llamada POST.
Realiza la consulta POST al endpoint /departamentos de georef-ar-api.
}
\examples{
\dontrun{
consultas_deptos <- list(
  list(provincia = "22", nombre = "Ledesma"),
  list(id = "14028")
)
resultados_deptos <- post_departamentos_bulk(queries_list = consultas_deptos)
print(resultados_deptos)
}
}
\references{
\href{https://datosgobar.github.io/georef-ar-api/open-api/#/Recursos/post_departamentos}{georef-ar-api/departamentos POST}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/georefar.R
\name{post_direcciones_bulk}
\alias{post_direcciones_bulk}
\title{Normalizar un Lote de Direcciones (POST)}
\usage{
post_direcciones_bulk(queries_list)
}
\arguments{
\item{queries_list}{Lista de listas. Cada lista interna debe contener los parámetros
para una consulta de normalización de dirección individual.
Parámetro requerido por consulta: 'direccion' (e.g., "AV SAN MARTIN 123").
Otros parámetros válidos: tipo, provincia, departamento, aplanar, campos, max, exacto.}
}
\value{
Un Data Frame (tibble) con los resultados combinados de todas las normalizaciones.
}
\description{
Permite normalizar múltiples direcciones en una sola llamada POST.
Realiza la consulta POST al endpoint /direcciones de georef-ar-api.
}
\examples{
\dontrun{
consultas_dir <- list(
  list(direccion = "MAIPU 100", provincia = "BUENOS AIRES"),
  list(direccion = "SANTA FE 2000, ROSARIO")
)
resultados_dir <- post_direcciones_bulk(queries_list = consultas_dir)
print(resultados_dir)
}
}
\references{
\href{https://datosgobar.github.io/georef-ar-api/open-api/#/Recursos/post_direcciones}{georef-ar-api/direcciones POST}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geo_metadata.R
\docType{data}
\name{geo_metadata}
\alias{geo_metadata}
\title{Un archivo de datos que contiene identificadores geográficos para los departamentos de Argentina y sus provincias
(\emph{A data file containing geographic identifiers for the departments of Argentina and their provinces})}
\format{
Un tibble con 526 filas y 8 variables:
\describe{
\item{codprov, coddepto}{identificadores utilizados por \emph{INDRA} para escrutinios provisorios de elecciones
nacionales}
\item{\emph{x_censo}}{identificadores de unidades geográficas del \emph{INDEC}}
\item{\emph{x_iso}}{identificadores de \emph{International Organization for Standardization}}

...
}
}
\usage{
geo_metadata
}
\description{
Un archivo de datos que contiene identificadores geográficos para los departamentos de Argentina y sus provincias
(\emph{A data file containing geographic identifiers for the departments of Argentina and their provinces})
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/georefar.R
\name{post_asentamientos_bulk}
\alias{post_asentamientos_bulk}
\title{Enviar Lote de Consultas de Asentamientos (POST)}
\usage{
post_asentamientos_bulk(queries_list)
}
\arguments{
\item{queries_list}{Lista de listas. Cada lista interna debe contener los parámetros
para una consulta de asentamiento individual.
Parámetros válidos por consulta: id, nombre, provincia, departamento, municipio, localidad_censal, orden, aplanar, campos, max, exacto.}
}
\value{
Un Data Frame (tibble) con los resultados combinados de todas las consultas.
}
\description{
Permite realizar múltiples búsquedas sobre el listado de asentamientos BAHRA en una sola llamada POST.
Realiza la consulta POST al endpoint /asentamientos de georef-ar-api.
}
\examples{
\dontrun{
consultas_asent <- list(
  list(provincia = "CHUBUT", nombre = "COSTA")
)
resultados_asent <- post_asentamientos_bulk(queries_list = consultas_asent)
print(resultados_asent)
}
}
\references{
\href{https://datosgobar.github.io/georef-ar-api/open-api/#/Recursos/post_asentamientos}{georef-ar-api/asentamientos POST}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add_geo_codes.R
\name{add_geo_codes}
\alias{add_geo_codes}
\title{Agrega columnas con id geográficos asignados a poligonos de provincias o departamentos de Argentina
(\emph{Augment Argentina's districts polygons id})}
\usage{
add_geo_codes(data)
}
\arguments{
\item{data}{data.frame obtenido con \code{\link{get_geo}}. Los mapas disponibles
se pueden chequear con \code{\link{show_arg_codes}}.}
}
\value{
Los valores por defecto en \code{\link{get_geo}} son \code{codprov_censo} para provincia y \code{coddepto_censo} para departamentos, respectivamente.
Estos corresponden a la codificación de INDEC.
Con el agregado usando \code{add_geo_codes} se puede optar por las nomenclaturas de \code{'indra'} - correspondiente a los escrutinios provisorios de elecciones nacionales,
con las variantes \code{codprov} y \code{coddepto}, o la de \code{'iso'} con \code{\*_iso} , estas últimas hasta el nivel
provincial.
}
\description{
Función que permite agregar columnas con ids geográficos para utilización de mapas de Argentina y de departamentos para los 24 distritos
sub nacionales para una más sencilla la vinculación con bases de datos de diversas fuentes.
}
\details{
Respecto el origen de los datos se puede consultar la documentación de
\href{https://www.iso.org/obp/ui/#iso:code:3166:AR}{\code{ISO 3166-2} - International Organization for Standardization} y
del \href{https://www.iso.org/obp/ui/#iso:code:3166:AR}{\emph{INDEC}}.

\code{codprov} y \code{coddepto} son las codificaciones de las bases de datos de \emph{INDRA}, empresa encargada por
muchos años de la tarea del escrutinio provisorio y utilizados en \href{https://electorarg.github.io/polAr/}{polAr}.
}
\examples{

get_geo("TUCUMAN")


get_geo("TUCUMAN") \%>\%
   add_geo_codes()


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/georefar.R
\name{normalizar_direccion}
\alias{normalizar_direccion}
\title{Normalizacion de direcciones}
\usage{
normalizar_direccion(
  direccion,
  provincia = NULL,
  departamento = NULL,
  localidad_censal = NULL,
  localidad = NULL,
  orden = NULL,
  aplanar = TRUE,
  campos = NULL,
  max = NULL,
  inicio = NULL,
  exacto = NULL
)
}
\arguments{
\item{direccion}{text Requerido. Direccion a normalizar, debe contener altura separada por espacio. (Ej: Colon 127)}

\item{provincia}{text Filtrar por nombre o ID de provincia.}

\item{departamento}{text Filtrar por nombre o ID de departamento.}

\item{localidad_censal}{text Filtrar por nombre o ID de localidad censal.}

\item{localidad}{text Filtrar por nombre o ID de localidad.}

\item{orden}{text Campo por el cual ordenar los resultados. (Por ID o nombre)}

\item{aplanar}{boolean Cuando está presente, muestra el resultado JSON con una estructura plana.}

\item{campos}{text Campos a incluir en la respuesta separados por comas, sin espacios. Algunos campos siempre serán incluidos, incluso si no se agregaron en la lista. Para incluir campos de sub-entidades, separar los nombres con un punto, por ejemplo: provincia.id.}

\item{max}{integer Cantidad máxima de resultados a devolver. La API limita a un máximo de 10 para este endpoint.}

\item{inicio}{integer Cantidad de resultados a omitir desde el principio.}

\item{exacto}{boolean Cuando está presente, se activa el modo de búsqueda por texto exacto. Sólo tiene efecto cuando se usan campos de búsqueda por texto (por ejemplo, nombre).}
}
\value{
Un Data Frame con el listado normalizado de de direcciones
}
\description{
Permite normalizar una dirección utilizando el listado de vías de circulación.
Realiza la consulta GET al endpoint /direcciones de georef-ar-api.
Si existe GEOREFAR_TOKEN en el Renviron lo usará para hacer la consulta.
}
\examples{
\dontrun{
normalizar_direccion(direccion = "Corrientes 1200, Rosario")
normalizar_direccion(direccion = "SAN MARTIN 100", provincia = "02", max = 5)
}
}
\references{
\href{https://datosgobar.github.io/georef-ar-api/open-api/#/Recursos/get_direcciones}{georef-ar-api/direcciones}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/georefar.R
\name{get_geodata_dump}
\alias{get_geodata_dump}
\title{Descargar Datos Geográficos Completos}
\usage{
get_geodata_dump(entidad, formato, path_to_save = NULL)
}
\arguments{
\item{entidad}{Cadena de texto. La entidad geográfica a descargar.
Valores posibles: "provincias", "departamentos", "municipios",
"localidades", "localidades-censales", "asentamientos", "calles", "cuadras".}

\item{formato}{Cadena de texto. El formato deseado para el archivo.
Valores posibles: "csv", "json", "geojson", "ndjson".}

\item{path_to_save}{Cadena de texto opcional. Ruta completa (incluyendo nombre de archivo y extensión)
donde guardar el archivo descargado. Si es NULL (por defecto), la función devolverá el contenido
parseado (para json/geojson/ndjson) o un data frame (para csv).
Si se especifica una ruta, la función guardará el archivo y devolverá la ruta del archivo guardado.}
}
\value{
Dependiendo de 'path_to_save' y 'formato':
- Si 'path_to_save' se especifica: la ruta al archivo guardado (invisiblemente).
- Si 'path_to_save' es NULL:
- Para "csv": un data.frame.
- Para "json", "geojson", "ndjson": una lista o estructura de R parseada desde JSON.
- Si la descarga o parseo falla, genera un error.
}
\description{
Permite descargar listados completos de entidades geográficas en diversos formatos.
Accede al endpoint de la georef-ar-api.
Si existe GEOREFAR_TOKEN en el Renviron lo usará para hacer la consulta (aunque generalmente no es necesario para estos endpoints públicos).
}
\examples{
\dontrun{
  # Obtener provincias en formato GeoJSON como objeto R
  provincias_geojson <- get_geodata_dump(entidad = "provincias", formato = "geojson")

  # Guardar departamentos en formato CSV
  get_geodata_dump(entidad = "departamentos", formato = "csv", path_to_save = "deptos.csv")
}
}
\references{
\href{https://datosgobar.github.io/georef-ar-api/open-api/#/Recursos/get__filename_}{georef-ar-api/descargas}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/recode_grid.R
\name{recode_grid}
\alias{recode_grid}
\title{Recodifica id de grillas asignados a  provincias o departamentos de Argentina
(\emph{Recode Argentina's districs id in grids})}
\usage{
recode_grid(data, type = NULL)
}
\arguments{
\item{data}{data.frame obtenido con \code{\link{get_grid}}. Las grillas disponibles
se pueden chequear con \code{\link{show_arg_codes}}.}

\item{type}{la variante del código que se quiere definir para la grilla. Las opciones son \code{'indra'}, \code{'indec'} o \code{'iso'}.}
}
\value{
Los valores por defecto en \code{\link{get_grid}} son \code{codprov} para provincia y \code{coddepto} para departamentos, respectivamente.
Estos corresponden a la codificación de los escrutinios provisorios de elecciones nacionales y se etiquetaron como \code{'indra'} .
Se puede optar por la nomenclatura de \code{'indec'}, con la familia \code{\*_censo}, para ambos niveles, o la de \code{'iso'} con \code{\*_iso} ,
para el nivel provincial.
}
\description{
Función que permite re codificar etiquetas para utilización de grillas de Argentina y de departamentos para los 24 distritos
sub nacionales. Ello permite hacer mas sencilla la vinculación con bases de datos de diversas fuentes.
}
\details{
Respecto el origen de los datos se puede consultar la documentación de
\href{https://www.iso.org/obp/ui/#iso:code:3166:AR}{\code{ISO 3166-2} - International Organization for Standardization} y
del \href{https://www.iso.org/obp/ui/#iso:code:3166:AR}{\emph{INDEC}}.

\code{codprov} y \code{coddepto} son las codificaciones de las bases de datos de \emph{INDRA}, empresa encargada por
muchos años de la tarea del escrutinio provisorio y utilizados en \href{https://electorarg.github.io/polAr/}{polAr}.
}
\examples{

get_grid("ARGENTINA")


get_grid("ARGENTINA") \%>\%
   recode_grid(type = "iso")


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/show_arg_codes.R
\name{show_arg_codes}
\alias{show_arg_codes}
\title{Diccionario de códigos identificadores de distritos
(\emph{geo ID's dictoniary})}
\usage{
show_arg_codes(viewer = FALSE, nivel = "provincias")
}
\arguments{
\item{viewer}{Por default es \code{TRUE} y  muestra una tabla formateada en el \emph{Viewer} de \emph{RStudio}. Cuando \code{FALSE} imprime en consola.}

\item{nivel}{Un character que permite elegir opción para ver diccionario a nivel de \code{"provincias"} o de \code{"departamentos"}.}
}
\value{
tibble con códigos geográficos correspondientes a distritos de Argentina descargados con \code{\link{get_geo}}
}
\description{
Función que devuelve un \emph{data.frame} con códigos y equivalencias de identificación de unidades geográficas
}
\examples{

show_arg_codes(viewer = FALSE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/georefar.R
\name{post_ubicacion_bulk}
\alias{post_ubicacion_bulk}
\title{Georreferenciación Inversa para un Lote de Puntos (POST)}
\usage{
post_ubicacion_bulk(queries_list)
}
\arguments{
\item{queries_list}{Lista de listas. Cada lista interna debe contener los parámetros
'lat' y 'lon' para un punto.
Otros parámetros válidos por consulta: aplanar, campos.}
}
\value{
Un Data Frame (tibble) con los resultados combinados de todas las georreferenciaciones.
}
\description{
Permite realizar georreferenciación inversa para múltiples puntos (lat, lon) en una sola llamada POST.
Realiza la consulta POST al endpoint /ubicacion de georef-ar-api.
}
\examples{
\dontrun{
consultas_ubic <- list(
  list(lat = -34.6037, lon = -58.3816), # Buenos Aires
  list(lat = -32.9587, lon = -60.6393)  # Rosario
)
resultados_ubic <- post_ubicacion_bulk(queries_list = consultas_ubic)
print(resultados_ubic)
}
}
\references{
\href{https://datosgobar.github.io/georef-ar-api/open-api/#/Recursos/post_ubicacion}{georef-ar-api/ubicacion POST}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/georefar.R
\name{post_municipios_bulk}
\alias{post_municipios_bulk}
\title{Enviar Lote de Consultas de Municipios (POST)}
\usage{
post_municipios_bulk(queries_list)
}
\arguments{
\item{queries_list}{Lista de listas. Cada lista interna debe contener los parámetros
para una consulta de municipio individual.
Parámetros válidos por consulta: id, nombre, provincia, departamento, orden, aplanar, campos, max, exacto.}
}
\value{
Un Data Frame (tibble) con los resultados combinados de todas las consultas.
}
\description{
Permite realizar múltiples búsquedas sobre el listado de municipios en una sola llamada POST.
Realiza la consulta POST al endpoint /municipios de georef-ar-api.
}
\examples{
\dontrun{
consultas_muni <- list(
  list(provincia = "06", nombre = "La Plata"),
  list(id = "540098")
)
resultados_muni <- post_municipios_bulk(queries_list = consultas_muni)
print(resultados_muni)
}
}
\references{
\href{https://datosgobar.github.io/georef-ar-api/open-api/#/Recursos/post_municipios}{georef-ar-api/municipios POST}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/addArgTiles.R
\name{addArgTiles}
\alias{addArgTiles}
\title{Agrega capa de Argentina para mapa interactivo
(\emph{Load Argentina Tiles Template})}
\usage{
addArgTiles(data)
}
\arguments{
\item{data}{debe ser un objeto con class "leaflet" "htmlwidget".}
}
\value{
Capa base para mapas interactivos
}
\description{
Función que descarga capa base de Argentina para mapa interactivo creado con la libreria \code{leaflet}.
}
\details{
El objetivo es agregar referencias de la base sobre la que se visualizan geometrías de Argentina. Se pueden agregar capas como polígonos descargandolos con \code{\link{get_geo}}.
Los geometrías disponibles se pueden chequear con \code{\link{show_arg_codes}}.
}
\examples{

 get_geo("TUCUMAN") \%>\%
 leaflet::leaflet() \%>\%
 leaflet::addPolygons() \%>\%
 addArgTiles()


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geoAr.R
\docType{package}
\name{geoAr}
\alias{geoAr-package}
\alias{geoAr}
\title{\code{geoAr} package}
\description{
Caja de Herramientas GEO de Argentina
See the README on
\href{https://github.com/PoliticaArgentina/geoAr#readme}{Github}
}
\seealso{
Useful links:
\itemize{
  \item \url{https://politicaargentina.github.io/geoAr/}
  \item \url{https://github.com/PoliticaArgentina/geoAr}
  \item Report bugs at \url{https://github.com/PoliticaArgentina/geoAr/issues}
}

}
\author{
\strong{Maintainer}: Juan Pablo Ruiz Nicolini \email{juanpabloruiznicolini@gmail.com} (\href{https://orcid.org/0000-0002-3138-6343}{ORCID}) [copyright holder]

Authors:
\itemize{
  \item Patricio Del Boca \email{patriciodelboca@gmail.com}
  \item Juan Gabriel Juara \email{jg.juara@gmail.com}
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/georefar.R
\name{replace_null_with_na}
\alias{replace_null_with_na}
\title{Replace NULL Values with NA}
\usage{
replace_null_with_na(x)
}
\arguments{
\item{x}{A list, vector, or single value to process}
}
\value{
The input with all NULL values replaced by NA
}
\description{
Recursively replaces NULL values with NA in a list or vector.
This function is useful for handling NULL values in API responses
before converting them to data frames.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_eph.R
\name{get_eph}
\alias{get_eph}
\title{Carga poligonos de los Aglomerados Urbanos correspondientes a la Encuesta Permanente de Hogares (INDEC)}
\usage{
get_eph(
  geo = "ARGENTINA",
  simplified = TRUE,
  centroid = FALSE,
  level = "envolventes"
)
}
\arguments{
\item{geo}{un character con el nombre del distrito que se quiere descargar.
Se puede chequear el id con \code{\link{show_arg_codes}}.}

\item{simplified}{por defecto es TRUE y determina la descarga de una versión simplificada de las geometrias.
Con FALSE descarga la versión original de INDEC}

\item{centroid}{por defecto devuelve poligonos como geometry pero pueden descargarse puntos (centroides correspondientes al level especificado)}

\item{level}{por defecto devuelve a nivel \code{envolvente} pero puede descargarse a nivel \code{radios} y \code{entidades}}
}
\value{
tibble con capa geografica correspondiente a alguna versión de geometrías utilizadas en la Encuesta Permanente de Hogares (EPH)
}
\description{
Carga poligonos de los Aglomerados Urbanos correspondientes a la Encuesta Permanente de Hogares (INDEC)
}
\examples{
get_eph(geo = "TUCUMAN")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_censo.R
\name{get_censo}
\alias{get_censo}
\title{Carga poligonos de Censos de Población Históricos de Argentina}
\usage{
get_censo(censo = NULL, simplified = FALSE)
}
\arguments{
\item{censo}{un character con el id del año del censo del district que se quiere descargar.
Se pueden chequear el id con \code{\link{show_available_censos}}.}

\item{simplified}{por defecto es TRUE y determina la descarga de una versión simplificada de las geometrias.
Con FALSE descarga la versión original de INDEC}
}
\value{
tibble con capa geografica correspondiente a geometrías de un censo histórico
}
\description{
Carga poligonos de Censos de Población Históricos de Argentina
}
\examples{
get_censo(censo = "1947", simplified = TRUE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/georefar.R
\name{get_ubicacion}
\alias{get_ubicacion}
\title{Obtener Ubicacion}
\usage{
get_ubicacion(lat, lon, aplanar = TRUE, campos = NULL)
}
\arguments{
\item{lat}{numeric Latitud del punto, en forma de número real con grados decimales.}

\item{lon}{numeric Longitud del punto, en forma de número real con grados decimales.}

\item{aplanar}{boolean Cuando está presente, muestra el resultado JSON con una estructura plana.}

\item{campos}{text Campos a incluir en la respuesta separados por comas, sin espacios. Algunos campos siempre serán incluidos, incluso si no se agregaron en la lista. Para incluir campos de sub-entidades, separar los nombres con un punto, por ejemplo: provincia.id.}
}
\value{
Un Data Frame con las unidades territoriales que contienen el punto.
}
\description{
Permite realizar una georreferenciación inversa para un punto, informando cuales unidades territoriales lo contienen.
Realiza la consulta GET al endpoint /ubicacion de georef-ar-api.
Si existe GEOREFAR_TOKEN en el Renviron lo usará para hacer la consulta.
}
\examples{
\dontrun{
get_ubicacion()
}
}
\references{
\href{https://datosgobar.github.io/georef-ar-api/open-api/#/Recursos/get_ubicacion}{georef-ar-api/ubicacion}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/show_available_censos.R
\name{show_available_censos}
\alias{show_available_censos}
\title{Geometrías de CENSOS
(\emph{geo ID's dictoniary})}
\usage{
show_available_censos()
}
\value{
tibble con información auxiliar para descarga de CENSOS históricos con  \code{\link{get_censo}}
}
\description{
Función que devuelve un \emph{data.frame} con listado de las geometrías de los CENSOS disponible
}
\examples{
show_available_censos()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/censos.R
\docType{data}
\name{censos}
\alias{censos}
\title{Un archivo de datos que contiene el listado de las geometrías para los censos disponibles}
\format{
Un tibble con 10 filas filas y 1 variables (censo)que funciona como id para usar con \code{\link{get_geo}}
\describe{
\item{censo}{ID correspondiente al año del censo de Argentina para descargar poligonos disponibles (1869 - 2010)

}}
}
\usage{
censos
}
\description{
Un archivo de datos que contiene el listado de las geometrías para los censos disponibles
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/georefar.R
\name{get_localidades}
\alias{get_localidades}
\title{Obtener Localidades}
\usage{
get_localidades(
  id = NULL,
  nombre = NULL,
  provincia = NULL,
  departamento = NULL,
  municipio = NULL,
  interseccion = NULL,
  orden = NULL,
  aplanar = TRUE,
  campos = NULL,
  max = NULL,
  inicio = NULL,
  exacto = NULL
)
}
\arguments{
\item{id}{text Filtrar por ID.}

\item{nombre}{text Filtrar por Nombre.}

\item{provincia}{text Filtrar por nombre o ID de Provincia.}

\item{departamento}{text Filtrar por nombre o ID de Departamento.}

\item{municipio}{text Filtrar por nombre o ID de Municipio.}

\item{interseccion}{text Geometría GeoJSON utilizada para filtrar resultados por intersección espacial. Sólo se soportan polígonos y multipolígonos. Ejemplo: polygon((-58.431,-34.592),(-58.430,-34.590),(-58.428,-34.593),(-58.431,-34.592)).}

\item{orden}{text Campo por el cual ordenar los resultados (por ID o nombre)}

\item{aplanar}{boolean Cuando está presente, muestra el resultado JSON con una estructura plana.}

\item{campos}{text Campos a incluir en la respuesta separados por comas, sin espacios. Algunos campos siempre serán incluidos, incluso si no se agregaron en la lista. Para incluir campos de sub-entidades, separar los nombres con un punto, por ejemplo: provincia.id.}

\item{max}{integer Cantidad máxima de resultados a devolver. La API limita a un máximo de 2000 para este endpoint.}

\item{inicio}{integer Cantidad de resultados a omitir desde el principio.}

\item{exacto}{boolean Cuando está presente, se activa el modo de búsqueda por texto exacto. Sólo tiene efecto cuando se usan campos de búsqueda por texto (por ejemplo, nombre).}
}
\value{
Un Data Frame con el listado de Localidades
}
\description{
Permite realizar búsquedas sobre el listado de localidades.
Realiza la consulta GET al endpoint /localidades de georef-ar-api.
Si existe GEOREFAR_TOKEN en el Renviron lo usará para hacer la consulta.
}
\examples{
\dontrun{
get_localidades(nombre = "PALERMO", provincia = "CIUDAD AUTONOMA DE BUENOS AIRES")
}
}
\references{
\href{https://datosgobar.github.io/georef-ar-api/open-api/#/Recursos/get_localidades}{georef-ar-api/localidades}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/georefar.R
\name{post_calles_bulk}
\alias{post_calles_bulk}
\title{Enviar Lote de Consultas de Calles (POST)}
\usage{
post_calles_bulk(queries_list)
}
\arguments{
\item{queries_list}{Lista de listas. Cada lista interna debe contener los parámetros
para una consulta de calle individual.
Parámetros válidos por consulta: nombre, id, tipo, provincia, departamento, municipio, localidad_censal, categoria, max, inicio, aplanar, campos, exacto.}
}
\value{
Un Data Frame (tibble) con los resultados combinados de todas las consultas.
}
\description{
Permite realizar múltiples búsquedas sobre el listado de vías de circulación en una sola llamada POST.
Realiza la consulta POST al endpoint /calles de georef-ar-api.
}
\examples{
\dontrun{
consultas_calles <- list(
  list(nombre = "SAN MARTIN", provincia = "BUENOS AIRES", max = 2),
  list(nombre = "SARMIENTO", provincia = "CORDOBA", max = 2)
)
resultados_calles <- post_calles_bulk(queries_list = consultas_calles)
print(resultados_calles)
}
}
\references{
\href{https://datosgobar.github.io/georef-ar-api/open-api/#/Recursos/post_calles}{georef-ar-api/calles POST}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_grid.R
\name{get_grid}
\alias{get_grid}
\title{Carga grillas de districts de Argentina
(\emph{Load grids of districts of Argentina})}
\usage{
get_grid(district = NULL)
}
\arguments{
\item{district}{un character con el nombre del district que se quiere descargar. Disponibles grillas para Argentina
y para las 24 provincias. Se pueden chequear los parametros con \code{\link{show_arg_codes}}.}
}
\value{
tibble con datos correspondientes a diseño de grillas para utilizar con libreria \code{geofacet}
}
\description{
Función que descarga grillas (\emph{facet}) para acomodarlas como si fueran mapas de provincias y deparamentos
de Argentina.
}
\examples{

get_grid("TUCUMAN")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/georefar.R
\name{post_localidades_bulk}
\alias{post_localidades_bulk}
\title{Enviar Lote de Consultas de Localidades (POST)}
\usage{
post_localidades_bulk(queries_list)
}
\arguments{
\item{queries_list}{Lista de listas. Cada lista interna debe contener los parámetros
para una consulta de localidad individual.
Parámetros válidos por consulta: id, nombre, provincia, departamento, municipio, orden, aplanar, campos, max, exacto.}
}
\value{
Un Data Frame (tibble) con los resultados combinados de todas las consultas.
}
\description{
Permite realizar múltiples búsquedas sobre el listado de localidades en una sola llamada POST.
Realiza la consulta POST al endpoint /localidades de georef-ar-api.
}
\examples{
\dontrun{
consultas_loc <- list(
  list(provincia = "Tucuman", departamento = "Capital"),
  list(id = "22056140000")
)
resultados_loc <- post_localidades_bulk(queries_list = consultas_loc)
print(resultados_loc)
}
}
\references{
\href{https://datosgobar.github.io/georef-ar-api/open-api/#/Recursos/post_localidades}{georef-ar-api/localidades POST}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_geo.R
\name{get_geo}
\alias{get_geo}
\title{Carga poligonos geográficos de distritos de Argentina
(\emph{Load Argentina's districts geometries})}
\usage{
get_geo(geo = NULL, level = "departamento", simplified = TRUE)
}
\arguments{
\item{geo}{un character con el nombre del district que se quiere descargar.
Se pueden chequear el id con \code{\link{show_arg_codes}}.}

\item{level}{parametro opcional para descargar geometrías a nivel 'departamento' o 'censal'
cuando se solicita mapa nacional \code{get_geo(geo = "ARGNTINA", level = "departamento")}.}

\item{simplified}{por defecto es TRUE y determina la descarga de una versión simplificada de las geometrias.
Con FALSE descarga la versión original de INDEC}
}
\value{
tibble con capa geografica de Argentina o distrito seleccionado (a diferentes niveles de agregación) correspondientes al CENSO 2010 - INDEC
}
\description{
Función que descarga (\emph{geometry}) para graficar con mapas
}
\examples{

get_geo("TUCUMAN")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grillas_geofacet.R
\docType{data}
\name{grillas_geofacet}
\alias{grillas_geofacet}
\title{Base de datos de grillas}
\format{
Una lista que contiene 25 \emph{data.frames} con 5 variables:
\describe{
\item{code_provincia}{identificación del distrito al que pertenece la grilla}
\item{col}{LONGITUD relativa de los distritos de segundo nivel en la grilla asemejando la geografía}
\item{row}{LATITUD relativa de los distritos de segundo nivel en la grilla asemejando la geografía}
\item{code}{código de identificación de cada uno de los distritos que componen la grilla}
\item{name}{nombre de cada uno de los distritos que componen la grilla}

...
}
}
\usage{
grillas_geofacet
}
\description{
Grillas de Argentina para usar con \code{facet_geo()}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_bahra.R
\name{get_bahra}
\alias{get_bahra}
\title{Descarga 'base total' de Asentamientos Humanos de la República Argentina (BAHRA)}
\usage{
get_bahra(geo = "ARGENTINA")
}
\arguments{
\item{geo}{un character con el nombre del distrito que se quiere descargar (por defecto toda ARGENTINA)
Se puede chequear el id con \code{\link{show_arg_codes}}.}
}
\value{
tibble con datos de BAHRA
}
\description{
Descarga 'base total' de Asentamientos Humanos de la República Argentina (BAHRA)
}
\examples{
get_bahra()
}
